# Copyright 2017 IBM Corp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import unittest
from unittest import mock

from oslo_concurrency import lockutils
from oslo_config import cfg
import testtools

from tempest import clients
from tempest import config
from tempest.lib.common import validation_resources as vr
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import base_compute_client
from tempest.lib.services.placement import base_placement_client
from tempest.lib.services.volume import base_client as base_volume_client
from tempest import test
from tempest.tests import base
from tempest.tests import fake_config
from tempest.tests.lib import fake_credentials
from tempest.tests.lib.services import registry_fixture

CONF = config.CONF


class LoggingTestResult(testtools.TestResult):

    def __init__(self, log, *args, **kwargs):
        super(LoggingTestResult, self).__init__(*args, **kwargs)
        self.log = log

    def addError(self, test, err=None, details=None):
        self.log.append((test, err, details))


class TestValidationResources(base.TestCase):

    validation_resources_module = 'tempest.lib.common.validation_resources'

    def setUp(self):
        super(TestValidationResources, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.useFixture(registry_fixture.RegistryFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)

        class TestTestClass(test.BaseTestCase):
            pass

        self.test_test_class = TestTestClass

    def test_validation_resources_no_validation(self):
        cfg.CONF.set_default('run_validation', False, 'validation')
        creds = fake_credentials.FakeKeystoneV3Credentials()
        osclients = clients.Manager(creds)
        vr = self.test_test_class.get_class_validation_resources(osclients)
        self.assertEqual({}, vr)

    def test_validation_resources_exists(self):
        cfg.CONF.set_default('run_validation', True, 'validation')
        creds = fake_credentials.FakeKeystoneV3Credentials()
        osclients = clients.Manager(creds)
        expected_vr = 'expected_validation_resources'
        self.test_test_class._validation_resources[osclients] = expected_vr
        obtained_vr = self.test_test_class.get_class_validation_resources(
            osclients)
        self.assertEqual(expected_vr, obtained_vr)

    @mock.patch(validation_resources_module + '.create_validation_resources',
                autospec=True)
    def test_validation_resources_new(self, mock_create_vr):
        cfg.CONF.set_default('run_validation', True, 'validation')
        cfg.CONF.set_default('neutron', True, 'service_available')
        creds = fake_credentials.FakeKeystoneV3Credentials()
        osclients = clients.Manager(creds)
        expected_vr = {'expected_validation_resources': None}
        mock_create_vr.return_value = expected_vr
        with mock.patch.object(
                self.test_test_class,
                'addClassResourceCleanup') as mock_add_class_cleanup:
            obtained_vr = self.test_test_class.get_class_validation_resources(
                osclients)
            self.assertEqual(1, mock_add_class_cleanup.call_count)
            self.assertEqual(mock.call(vr.clear_validation_resources,
                                       osclients,
                                       use_neutron=True,
                                       **expected_vr),
                             mock_add_class_cleanup.call_args)
        self.assertEqual(mock_create_vr.call_count, 1)
        self.assertIn(osclients, mock_create_vr.call_args_list[0][0])
        self.assertEqual(expected_vr, obtained_vr)
        self.assertIn(osclients, self.test_test_class._validation_resources)
        self.assertEqual(expected_vr,
                         self.test_test_class._validation_resources[osclients])

    def test_validation_resources_invalid_config(self):
        invalid_version = 999
        cfg.CONF.set_default('run_validation', True, 'validation')
        cfg.CONF.set_default('ip_version_for_ssh', invalid_version,
                             'validation')
        cfg.CONF.set_default('neutron', True, 'service_available')
        creds = fake_credentials.FakeKeystoneV3Credentials()
        osclients = clients.Manager(creds)
        with testtools.ExpectedException(
                lib_exc.InvalidConfiguration,
                value_re='^.*\n.*' + str(invalid_version)):
            self.test_test_class.get_class_validation_resources(osclients)

    @mock.patch(validation_resources_module + '.create_validation_resources',
                autospec=True)
    def test_validation_resources_invalid_config_nova_net(self,
                                                          mock_create_vr):
        invalid_version = 999
        cfg.CONF.set_default('run_validation', True, 'validation')
        cfg.CONF.set_default('ip_version_for_ssh', invalid_version,
                             'validation')
        cfg.CONF.set_default('neutron', False, 'service_available')
        creds = fake_credentials.FakeKeystoneV3Credentials()
        osclients = clients.Manager(creds)
        expected_vr = {'expected_validation_resources': None}
        mock_create_vr.return_value = expected_vr
        obtained_vr = self.test_test_class.get_class_validation_resources(
            osclients)
        self.assertEqual(mock_create_vr.call_count, 1)
        self.assertIn(osclients, mock_create_vr.call_args_list[0][0])
        self.assertEqual(expected_vr, obtained_vr)
        self.assertIn(osclients, self.test_test_class._validation_resources)
        self.assertEqual(expected_vr,
                         self.test_test_class._validation_resources[osclients])

    @mock.patch(validation_resources_module + '.create_validation_resources',
                autospec=True)
    @mock.patch(validation_resources_module + '.clear_validation_resources',
                autospec=True)
    def test_validation_resources_fixture(self, mock_clean_vr, mock_create_vr):

        class TestWithRun(self.test_test_class):

            def runTest(self):
                pass

        cfg.CONF.set_default('run_validation', True, 'validation')
        test_case = TestWithRun()
        creds = fake_credentials.FakeKeystoneV3Credentials()
        osclients = clients.Manager(creds)
        test_case.get_test_validation_resources(osclients)
        self.assertEqual(1, mock_create_vr.call_count)
        self.assertEqual(0, mock_clean_vr.call_count)


class TestSetNetworkResources(base.TestCase):

    def setUp(self):
        super(TestSetNetworkResources, self).setUp()

        class ParentTest(test.BaseTestCase):

            @classmethod
            def setup_credentials(cls):
                cls.set_network_resources(dhcp=True)
                super(ParentTest, cls).setup_credentials()

            def runTest(self):
                pass

        self.parent_class = ParentTest

    def test_set_network_resources_child_only(self):

        class ChildTest(self.parent_class):

            @classmethod
            def setup_credentials(cls):
                cls.set_network_resources(router=True)
                super(ChildTest, cls).setup_credentials()

        child_test = ChildTest()
        child_test.setUpClass()
        # Assert that the parents network resources are not set
        self.assertFalse(child_test._network_resources['dhcp'])
        # Assert that the child network resources are set
        self.assertTrue(child_test._network_resources['router'])

    def test_set_network_resources_right_order(self):

        class ChildTest(self.parent_class):

            @classmethod
            def setup_credentials(cls):
                super(ChildTest, cls).setup_credentials()
                cls.set_network_resources(router=True)

        child_test = ChildTest()
        with testtools.ExpectedException(RuntimeError,
                                         value_re='set_network_resources'):
            child_test.setUpClass()

    def test_set_network_resources_children(self):

        class ChildTest(self.parent_class):

            @classmethod
            def setup_credentials(cls):
                cls.set_network_resources(router=True)
                super(ChildTest, cls).setup_credentials()

        class GrandChildTest(ChildTest):
            pass

        # Invoke setupClass on both and check that the setup_credentials
        # call check mechanism does not report any false negative.
        child_test = ChildTest()
        child_test.setUpClass()
        grandchild_test = GrandChildTest()
        grandchild_test.setUpClass()


class TestTempestBaseTestClass(base.TestCase):

    def setUp(self):
        super(TestTempestBaseTestClass, self).setUp()
        self.useFixture(fake_config.ConfigFixture())
        self.patchobject(config, 'TempestConfigPrivate',
                         fake_config.FakePrivate)

        class ParentTest(test.BaseTestCase):

            def runTest(self):
                pass

        self.parent_test = ParentTest

    def test_resource_cleanup(self):
        cfg.CONF.set_default('neutron', False, 'service_available')
        exp_args = (1, 2,)
        exp_kwargs = {'a': 1, 'b': 2}
        mock1 = mock.Mock()
        mock2 = mock.Mock()
        exp_functions = [mock1, mock2]

        class TestWithCleanups(self.parent_test):

            @classmethod
            def resource_setup(cls):
                for fn in exp_functions:
                    cls.addClassResourceCleanup(fn, *exp_args,
                                                **exp_kwargs)

        test_cleanups = TestWithCleanups()
        suite = unittest.TestSuite((test_cleanups,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # No exception raised - error log is empty
        self.assertFalse(log)
        # All stacked resource cleanups invoked
        mock1.assert_called_once_with(*exp_args, **exp_kwargs)
        mock2.assert_called_once_with(*exp_args, **exp_kwargs)
        # Cleanup stack is empty
        self.assertEqual(0, len(test_cleanups._class_cleanups))

    def test_resource_cleanup_failures(self):
        cfg.CONF.set_default('neutron', False, 'service_available')
        exp_args = (1, 2,)
        exp_kwargs = {'a': 1, 'b': 2}
        mock1 = mock.Mock()
        mock1.side_effect = Exception('mock1 resource cleanup failure')
        mock2 = mock.Mock()
        mock3 = mock.Mock()
        mock3.side_effect = Exception('mock3 resource cleanup failure')
        exp_functions = [mock1, mock2, mock3]

        class TestWithFailingCleanups(self.parent_test):

            @classmethod
            def resource_setup(cls):
                for fn in exp_functions:
                    cls.addClassResourceCleanup(fn, *exp_args,
                                                **exp_kwargs)

        test_cleanups = TestWithFailingCleanups()
        suite = unittest.TestSuite((test_cleanups,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # One multiple exception captured
        self.assertEqual(1, len(log))
        # [0]: test, err, details [1] -> exc_info
        # Type, Exception, traceback [1] -> MultipleException
        found_exc = log[0][1][1]
        self.assertIsInstance(found_exc, testtools.MultipleExceptions)
        self.assertEqual(2, len(found_exc.args))
        # Each arg is exc_info - match messages and order
        self.assertIn('mock3 resource', str(found_exc.args[0][1]))
        self.assertIn('mock1 resource', str(found_exc.args[1][1]))
        # All stacked resource cleanups invoked
        mock1.assert_called_once_with(*exp_args, **exp_kwargs)
        mock2.assert_called_once_with(*exp_args, **exp_kwargs)
        # Cleanup stack is empty
        self.assertEqual(0, len(test_cleanups._class_cleanups))

    def test_super_resource_cleanup_not_invoked(self):

        class BadResourceCleanup(self.parent_test):

            @classmethod
            def resource_cleanup(cls):
                pass

        bad_class = BadResourceCleanup()
        suite = unittest.TestSuite((bad_class,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # One multiple exception captured
        self.assertEqual(1, len(log))
        # [0]: test, err, details [1] -> exc_info
        # Type, Exception, traceback [1] -> RuntimeError
        found_exc = log[0][1][1]
        self.assertIsInstance(found_exc, RuntimeError)
        self.assertIn(BadResourceCleanup.__name__, str(found_exc))

    def test_super_skip_checks_not_invoked(self):

        class BadSkipChecks(self.parent_test):

            @classmethod
            def skip_checks(cls):
                pass

        bad_class = BadSkipChecks()
        with testtools.ExpectedException(
                RuntimeError,
                value_re='^.* ' + BadSkipChecks.__name__):
            bad_class.setUpClass()

    def test_super_setup_credentials_not_invoked(self):

        class BadSetupCredentials(self.parent_test):

            @classmethod
            def skip_checks(cls):
                pass

        bad_class = BadSetupCredentials()
        with testtools.ExpectedException(
                RuntimeError,
                value_re='^.* ' + BadSetupCredentials.__name__):
            bad_class.setUpClass()

    def test_grandparent_skip_checks_not_invoked(self):

        class BadSkipChecks(self.parent_test):

            @classmethod
            def skip_checks(cls):
                pass

        class SonOfBadSkipChecks(BadSkipChecks):
            pass

        bad_class = SonOfBadSkipChecks()
        with testtools.ExpectedException(
                RuntimeError,
                value_re='^.* ' + SonOfBadSkipChecks.__name__):
            bad_class.setUpClass()

    @mock.patch('tempest.common.credentials_factory.is_admin_available',
                autospec=True, return_value=True)
    def test_skip_checks_admin(self, mock_iaa):
        identity_version = 'identity_version'

        class NeedAdmin(self.parent_test):
            credentials = ['admin']

            @classmethod
            def get_identity_version(cls):
                return identity_version

        NeedAdmin().skip_checks()
        mock_iaa.assert_called_once_with('identity_version')

    @mock.patch('tempest.common.credentials_factory.is_admin_available',
                autospec=True, return_value=False)
    def test_skip_checks_admin_not_available(self, mock_iaa):
        identity_version = 'identity_version'

        class NeedAdmin(self.parent_test):
            credentials = ['admin']

            @classmethod
            def get_identity_version(cls):
                return identity_version

        with testtools.ExpectedException(testtools.testcase.TestSkipped):
            NeedAdmin().skip_checks()
        mock_iaa.assert_called_once_with('identity_version')

    def test_skip_checks_identity_v2_not_available(self):
        cfg.CONF.set_default('api_v2', False, 'identity-feature-enabled')

        class NeedV2(self.parent_test):
            identity_version = 'v2'

        with testtools.ExpectedException(testtools.testcase.TestSkipped):
            NeedV2().skip_checks()

    def test_skip_checks_identity_v3_not_available(self):
        cfg.CONF.set_default('api_v3', False, 'identity-feature-enabled')

        class NeedV3(self.parent_test):
            identity_version = 'v3'

        with testtools.ExpectedException(testtools.testcase.TestSkipped):
            NeedV3().skip_checks()

    def test_setup_credentials_all(self):
        expected_creds = ['string', ['list', 'role1', 'role2']]

        class AllCredentials(self.parent_test):
            credentials = expected_creds

        expected_clients = 'clients'
        with mock.patch.object(
                AllCredentials,
                'get_client_manager') as mock_get_client_manager:
            mock_get_client_manager.return_value = expected_clients
            all_creds = AllCredentials()
            all_creds.setup_credentials()
        self.assertTrue(hasattr(all_creds, 'os_string'))
        self.assertEqual(expected_clients, all_creds.os_string)
        self.assertTrue(hasattr(all_creds, 'os_roles_list'))
        self.assertEqual(expected_clients, all_creds.os_roles_list)
        self.assertEqual(2, mock_get_client_manager.call_count)
        self.assertEqual(
            expected_creds[0],
            mock_get_client_manager.mock_calls[0][2]['credential_type'])
        self.assertEqual(
            expected_creds[1][1:],
            mock_get_client_manager.mock_calls[1][2]['roles'])

    def test_setup_credentials_with_role_and_system_scope(self):
        expected_creds = [['system_my_role', 'role1', 'role2']]

        class SystemRoleCredentials(self.parent_test):
            credentials = expected_creds

        expected_clients = 'clients'
        with mock.patch.object(
                SystemRoleCredentials,
                'get_client_manager') as mock_get_client_manager:
            mock_get_client_manager.return_value = expected_clients
            sys_creds = SystemRoleCredentials()
            sys_creds.setup_credentials()
        self.assertTrue(hasattr(sys_creds, 'os_system_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_system_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_roles_system_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_roles_system_my_role)
        self.assertEqual(1, mock_get_client_manager.call_count)
        self.assertEqual(
            expected_creds[0][1:],
            mock_get_client_manager.mock_calls[0][2]['roles'])
        self.assertEqual(
            'system',
            mock_get_client_manager.mock_calls[0][2]['scope'])

    def test_setup_credentials_with_multiple_role_and_system_scope(self):
        expected_creds = [['system_my_role', 'role1', 'role2'],
                          ['system_my_role2', 'role1', 'role2'],
                          ['system_my_role3', 'role3']]

        class SystemRoleCredentials(self.parent_test):
            credentials = expected_creds

        expected_clients = 'clients'
        with mock.patch.object(
                SystemRoleCredentials,
                'get_client_manager') as mock_get_client_manager:
            mock_get_client_manager.return_value = expected_clients
            sys_creds = SystemRoleCredentials()
            sys_creds.setup_credentials()
        self.assertTrue(hasattr(sys_creds, 'os_system_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_system_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_roles_system_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_roles_system_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_system_my_role2'))
        self.assertEqual(expected_clients, sys_creds.os_system_my_role2)
        self.assertTrue(hasattr(sys_creds, 'os_roles_system_my_role2'))
        self.assertEqual(expected_clients, sys_creds.os_roles_system_my_role2)
        self.assertTrue(hasattr(sys_creds, 'os_system_my_role3'))
        self.assertEqual(expected_clients, sys_creds.os_system_my_role3)
        self.assertTrue(hasattr(sys_creds, 'os_roles_system_my_role3'))
        self.assertEqual(expected_clients, sys_creds.os_roles_system_my_role3)
        self.assertEqual(3, mock_get_client_manager.call_count)
        self.assertEqual(
            expected_creds[0][1:],
            mock_get_client_manager.mock_calls[0][2]['roles'])
        self.assertEqual(
            'system', mock_get_client_manager.mock_calls[0][2]['scope'])
        self.assertEqual(
            expected_creds[1][1:],
            mock_get_client_manager.mock_calls[1][2]['roles'])
        self.assertEqual(
            'system', mock_get_client_manager.mock_calls[1][2]['scope'])
        self.assertEqual(
            expected_creds[2][1:],
            mock_get_client_manager.mock_calls[2][2]['roles'])
        self.assertEqual(
            'system', mock_get_client_manager.mock_calls[2][2]['scope'])

    def test_setup_credentials_with_role_and_multiple_scope(self):
        expected_creds = [['my_role', 'role1', 'role2'],
                          ['project_my_role', 'role1', 'role2'],
                          ['domain_my_role', 'role1', 'role2'],
                          ['system_my_role', 'role1', 'role2']]

        class SystemRoleCredentials(self.parent_test):
            credentials = expected_creds

        expected_clients = 'clients'
        with mock.patch.object(
                SystemRoleCredentials,
                'get_client_manager') as mock_get_client_manager:
            mock_get_client_manager.return_value = expected_clients
            sys_creds = SystemRoleCredentials()
            sys_creds.setup_credentials()
        self.assertTrue(hasattr(sys_creds, 'os_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_roles_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_roles_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_project_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_project_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_roles_project_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_roles_project_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_domain_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_domain_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_roles_domain_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_roles_domain_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_system_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_system_my_role)
        self.assertTrue(hasattr(sys_creds, 'os_roles_system_my_role'))
        self.assertEqual(expected_clients, sys_creds.os_roles_system_my_role)

        self.assertEqual(4, mock_get_client_manager.call_count)
        self.assertEqual(
            expected_creds[0][1:],
            mock_get_client_manager.mock_calls[0][2]['roles'])
        self.assertEqual(
            'project', mock_get_client_manager.mock_calls[0][2]['scope'])
        self.assertEqual(
            expected_creds[1][1:],
            mock_get_client_manager.mock_calls[1][2]['roles'])
        self.assertEqual(
            'project', mock_get_client_manager.mock_calls[1][2]['scope'])
        self.assertEqual(
            expected_creds[2][1:],
            mock_get_client_manager.mock_calls[2][2]['roles'])
        self.assertEqual(
            'domain', mock_get_client_manager.mock_calls[2][2]['scope'])
        self.assertEqual(
            expected_creds[3][1:],
            mock_get_client_manager.mock_calls[3][2]['roles'])
        self.assertEqual(
            'system', mock_get_client_manager.mock_calls[3][2]['scope'])

    def test_setup_class_overwritten(self):

        class OverridesSetup(self.parent_test):

            @classmethod
            def setUpClass(cls):  # noqa
                pass

        overrides_setup = OverridesSetup()
        suite = unittest.TestSuite((overrides_setup,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # Record 0, test (error holder). The error generates during test run.
        self.assertIn('runTest', str(log[0][0]))
        # Record 0, traceback
        self.assertRegex(
            str(log[0][2]['traceback']).replace('\n', ' '),
            RuntimeError.__name__ + ': .* ' + OverridesSetup.__name__)

    @mock.patch.object(test.process_lock, 'InterProcessReaderWriterLock')
    def test_serial_execution_if_requested(self, mock_lock):

        @decorators.serial
        class SerialTests(self.parent_test):
            pass

        class ParallelTests(self.parent_test):
            pass

        @decorators.serial
        class SerialTests2(self.parent_test):
            pass

        suite = unittest.TestSuite(
            (SerialTests(), ParallelTests(), SerialTests2()))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)

        expected_lock_path = os.path.join(
            lockutils.get_lock_path(CONF), 'tempest-serial-rw-lock')

        # We except that each test class has a lock with the _same_ external
        # path so that if they would run by different processes they would
        # still use the same lock
        # Also we expect that each serial class takes and releases the
        # write-lock while each non-serial class takes and releases the
        # read-lock.
        self.assertEqual(
            [
                mock.call(expected_lock_path),
                mock.call().acquire_write_lock(),
                mock.call().release_write_lock(),

                mock.call(expected_lock_path),
                mock.call().acquire_read_lock(),
                mock.call().release_read_lock(),

                mock.call(expected_lock_path),
                mock.call().acquire_write_lock(),
                mock.call().release_write_lock(),
            ],
            mock_lock.mock_calls
        )


class TestTempestBaseTestClassFixtures(base.TestCase):

    SETUP_FIXTURES = [test.BaseTestCase.setUpClass.__name__,
                      test.BaseTestCase.skip_checks.__name__,
                      test.BaseTestCase.setup_credentials.__name__,
                      test.BaseTestCase.setup_clients.__name__,
                      test.BaseTestCase.resource_setup.__name__]
    TEARDOWN_FIXTURES = [test.BaseTestCase.tearDownClass.__name__,
                         test.BaseTestCase.resource_cleanup.__name__,
                         test.BaseTestCase.clear_credentials.__name__]

    def setUp(self):
        super(TestTempestBaseTestClassFixtures, self).setUp()
        self.mocks = {}
        for fix in self.SETUP_FIXTURES + self.TEARDOWN_FIXTURES:
            self.mocks[fix] = mock.Mock()

        def tracker_builder(name):

            def tracker(cls):
                # Track that the fixture was invoked
                cls.fixtures_invoked.append(name)
                # Run the fixture
                getattr(super(TestWithClassFixtures, cls), name)()
                # Run a mock we can use for side effects
                self.mocks[name]()

            return tracker

        class TestWithClassFixtures(test.BaseTestCase):

            credentials = []
            fixtures_invoked = []

            def runTest(_self):
                pass

        # Decorate all test class fixtures with tracker_builder
        for method_name in self.SETUP_FIXTURES + self.TEARDOWN_FIXTURES:
            setattr(TestWithClassFixtures, method_name,
                    classmethod(tracker_builder(method_name)))

        self.test = TestWithClassFixtures()

    def test_no_error_flow(self):
        # If all setup fixtures are executed, all cleanup fixtures are
        # executed too
        suite = unittest.TestSuite((self.test,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        self.assertEqual(self.SETUP_FIXTURES + self.TEARDOWN_FIXTURES,
                         self.test.fixtures_invoked)

    def test_skip_only(self):
        # If a skip condition is hit in the test, no credentials or resource
        # is provisioned / cleaned-up
        self.mocks['skip_checks'].side_effect = (
            testtools.TestCase.skipException())
        suite = unittest.TestSuite((self.test,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # If we trigger a skip condition, teardown is not invoked at all
        self.assertEqual((self.SETUP_FIXTURES[:2] +
                          [self.TEARDOWN_FIXTURES[0]]),
                         self.test.fixtures_invoked)

    def test_skip_credentials_fails(self):
        expected_exc = 'sc exploded'
        self.mocks['setup_credentials'].side_effect = Exception(expected_exc)
        suite = unittest.TestSuite((self.test,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # If setup_credentials explodes, we invoked teardown class and
        # clear credentials, and re-raise
        self.assertEqual((self.SETUP_FIXTURES[:3] +
                          [self.TEARDOWN_FIXTURES[i] for i in (0, 2)]),
                         self.test.fixtures_invoked)
        found_exc = log[0][1][1]
        self.assertIn(expected_exc, str(found_exc))

    def test_skip_credentials_fails_clear_fails(self):
        # If cleanup fails on failure, we log the exception and do not
        # re-raise it. Note that since the exception happens outside of
        # the Tempest test setUp, logging is not captured on the Tempest
        # test side, it will be captured by the unit test instead.
        expected_exc = 'sc exploded'
        clear_exc = 'clear exploded'
        self.mocks['setup_credentials'].side_effect = Exception(expected_exc)
        self.mocks['clear_credentials'].side_effect = Exception(clear_exc)
        suite = unittest.TestSuite((self.test,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # If setup_credentials explodes, we invoked teardown class and
        # clear credentials, and re-raise
        self.assertEqual((self.SETUP_FIXTURES[:3] +
                          [self.TEARDOWN_FIXTURES[i] for i in (0, 2)]),
                         self.test.fixtures_invoked)
        found_exc = log[0][1][1]
        self.assertIn(expected_exc, str(found_exc))
        # Since log capture depends on OS_LOG_CAPTURE, we can only assert if
        # logging was captured
        if os.environ.get('OS_LOG_CAPTURE'):
            self.assertIn(clear_exc, self.log_fixture.logger.output)

    def test_skip_credentials_clients_resources_credentials_clear_fails(self):
        # If cleanup fails with no previous failure, we re-raise the exception.
        expected_exc = 'clear exploded'
        self.mocks['clear_credentials'].side_effect = Exception(expected_exc)
        suite = unittest.TestSuite((self.test,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # If setup_credentials explodes, we invoked teardown class and
        # clear credentials, and re-raise
        self.assertEqual(self.SETUP_FIXTURES + self.TEARDOWN_FIXTURES,
                         self.test.fixtures_invoked)
        found_exc = log[0][1][1]
        self.assertIn(expected_exc, str(found_exc))

    def test_skip_credentials_clients_fails(self):
        expected_exc = 'clients exploded'
        self.mocks['setup_clients'].side_effect = Exception(expected_exc)
        suite = unittest.TestSuite((self.test,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # If setup_clients explodes, we invoked teardown class and
        # clear credentials, and re-raise
        self.assertEqual((self.SETUP_FIXTURES[:4] +
                          [self.TEARDOWN_FIXTURES[i] for i in (0, 2)]),
                         self.test.fixtures_invoked)
        found_exc = log[0][1][1]
        self.assertIn(expected_exc, str(found_exc))

    def test_skip_credentials_clients_resources_fails(self):
        expected_exc = 'resource setup exploded'
        self.mocks['resource_setup'].side_effect = Exception(expected_exc)
        suite = unittest.TestSuite((self.test,))
        log = []
        result = LoggingTestResult(log)
        suite.run(result)
        # If resource_setup explodes, we invoked teardown class and
        # clear credentials and resource cleanup, and re-raise
        self.assertEqual(self.SETUP_FIXTURES + self.TEARDOWN_FIXTURES,
                         self.test.fixtures_invoked)
        found_exc = log[0][1][1]
        self.assertIn(expected_exc, str(found_exc))


class TestAPIMicroversionTest1(test.BaseTestCase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.useFixture(fake_config.ConfigFixture())
        config.TempestConfigPrivate = fake_config.FakePrivate

    @classmethod
    def resource_setup(cls):
        super(TestAPIMicroversionTest1, cls).resource_setup()
        # Setting microvesions and checks that every tests
        # of this class will have those microversion set
        # on service clients requesting service APIs.
        cls.setup_api_microversion_fixture(
            compute_microversion='2.30',
            volume_microversion='3.10',
            placement_microversion='3.1')
        # Check microvesion is set during resource_setup()
        if base_compute_client.COMPUTE_MICROVERSION != '2.30':
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")
        if base_volume_client.VOLUME_MICROVERSION != '3.10':
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")
        if base_placement_client.PLACEMENT_MICROVERSION != '3.1':
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")

    @classmethod
    def resource_cleanup(cls):
        super(TestAPIMicroversionTest1, cls).resource_cleanup()
        # Check microversion is reset back to None in resource_cleanup()
        if base_compute_client.COMPUTE_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")
        if base_volume_client.VOLUME_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")
        if base_placement_client.PLACEMENT_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")

    def setUp(self):
        super(TestAPIMicroversionTest1, self).setUp()
        # Check microversion is set in setUp method also.
        self.assertEqual('2.30', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.10', base_volume_client.VOLUME_MICROVERSION)
        self.assertEqual('3.1', base_placement_client.PLACEMENT_MICROVERSION)

    def tearDown(self):
        super(TestAPIMicroversionTest1, self).tearDown()
        # Check microversion is set in tearDown method also.
        self.assertEqual('2.30', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.10', base_volume_client.VOLUME_MICROVERSION)
        self.assertEqual('3.1', base_placement_client.PLACEMENT_MICROVERSION)

    def test_1(self):
        self.assertEqual('2.30', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.10', base_volume_client.VOLUME_MICROVERSION)
        self.assertEqual('3.1', base_placement_client.PLACEMENT_MICROVERSION)

    def test_2(self):
        self.assertEqual('2.30', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.10', base_volume_client.VOLUME_MICROVERSION)
        self.assertEqual('3.1', base_placement_client.PLACEMENT_MICROVERSION)


class TestAPIMicroversionTest2(test.BaseTestCase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.useFixture(fake_config.ConfigFixture())
        config.TempestConfigPrivate = fake_config.FakePrivate

    @classmethod
    def resource_setup(cls):
        super(TestAPIMicroversionTest2, cls).resource_setup()
        # Setting microvesions different from what set in
        # MicroversionTest1 and checks that every tests
        # of this class will have the new microversion set
        # on service clients requesting service APIs.
        cls.setup_api_microversion_fixture(
            compute_microversion='2.80',
            volume_microversion='3.80',
            placement_microversion='3.8')
        # Check microvesion is set during resource_setup()
        if base_compute_client.COMPUTE_MICROVERSION != '2.80':
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")
        if base_volume_client.VOLUME_MICROVERSION != '3.80':
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")
        if base_placement_client.PLACEMENT_MICROVERSION != '3.8':
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")

    @classmethod
    def resource_cleanup(cls):
        super(TestAPIMicroversionTest2, cls).resource_cleanup()
        # Check microversion is reset back to None in resource_cleanup()
        if base_compute_client.COMPUTE_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")
        if base_volume_client.VOLUME_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")
        if base_placement_client.PLACEMENT_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")

    def setUp(self):
        super(TestAPIMicroversionTest2, self).setUp()
        # Check microversion is set in setUp method also.
        self.assertEqual('2.80', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.80', base_volume_client.VOLUME_MICROVERSION)
        self.assertEqual('3.8', base_placement_client.PLACEMENT_MICROVERSION)

    def tearDown(self):
        super(TestAPIMicroversionTest2, self).tearDown()
        # Check microversion is set in tearDown method also.
        self.assertEqual('2.80', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.80', base_volume_client.VOLUME_MICROVERSION)
        self.assertEqual('3.8', base_placement_client.PLACEMENT_MICROVERSION)

    def test_1(self):
        self.assertEqual('2.80', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.80', base_volume_client.VOLUME_MICROVERSION)
        self.assertEqual('3.8', base_placement_client.PLACEMENT_MICROVERSION)

    def test_2(self):
        self.assertEqual('2.80', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.80', base_volume_client.VOLUME_MICROVERSION)
        self.assertEqual('3.8', base_placement_client.PLACEMENT_MICROVERSION)


class TestAPIMicroversionTest3(test.BaseTestCase):

    @classmethod
    def resource_setup(cls):
        super(TestAPIMicroversionTest3, cls).resource_setup()
        # Not setting microversion for this test class so
        # there should not be any micorversion set on service
        # clients requesting services APIs.
        # Check microvesion is not set during resource_setup()
        if base_compute_client.COMPUTE_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")
        if base_volume_client.VOLUME_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")
        if base_placement_client.PLACEMENT_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not set in resource_setup method")

    @classmethod
    def resource_cleanup(cls):
        super(TestAPIMicroversionTest3, cls).resource_cleanup()
        # Check microversion is set to None in resource_cleanup()
        if base_compute_client.COMPUTE_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")
        if base_volume_client.VOLUME_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")
        if base_placement_client.PLACEMENT_MICROVERSION is not None:
            raise testtools.TestCase.failureException(
                "Microversion is not reset to None in resource_cleanup method")

    def setUp(self):
        super(TestAPIMicroversionTest3, self).setUp()
        # Check microversion is None in setUp method also.
        self.assertIsNone(base_compute_client.COMPUTE_MICROVERSION)
        self.assertIsNone(base_volume_client.VOLUME_MICROVERSION)
        self.assertIsNone(base_placement_client.PLACEMENT_MICROVERSION)

    def tearDown(self):
        super(TestAPIMicroversionTest3, self).tearDown()
        # Check microversion is None in tearDown method also.
        self.assertIsNone(base_compute_client.COMPUTE_MICROVERSION)
        self.assertIsNone(base_volume_client.VOLUME_MICROVERSION)
        self.assertIsNone(base_placement_client.PLACEMENT_MICROVERSION)

    def test_1(self):
        self.assertIsNone(base_compute_client.COMPUTE_MICROVERSION)
        self.assertIsNone(base_volume_client.VOLUME_MICROVERSION)
        self.assertIsNone(base_placement_client.PLACEMENT_MICROVERSION)

    def test_2(self):
        self.assertIsNone(base_compute_client.COMPUTE_MICROVERSION)
        self.assertIsNone(base_volume_client.VOLUME_MICROVERSION)
        self.assertIsNone(base_placement_client.PLACEMENT_MICROVERSION)



# Copyright 2016 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import versions_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestNetworkVersionsClient(base.BaseServiceTest):
    VERSION = "v2.0"

    FAKE_VERSIONS_INFO = {
        "versions": [
            {
                "id": "v2.0",
                "links": [
                    {
                        "href": "http://openstack.example.com/%s/" % VERSION,
                        "rel": "self"
                    }
                ],
                "status": "CURRENT"
            }
        ]
    }

    FAKE_VERSION_DETAILS = {
        "resources": [
            {
                "collection": "subnets",
                "links": [
                    {
                        "href": "http://openstack.example.com:9696/"
                                "%s/subnets" % VERSION,
                        "rel": "self"
                    }
                ],
                "name": "subnet"
            },
            {
                "collection": "networks",
                "links": [
                    {
                        "href": "http://openstack.example.com:9696/"
                                "%s/networks" % VERSION,
                        "rel": "self"
                    }
                ],
                "name": "network"
            },
            {
                "collection": "ports",
                "links": [
                    {
                        "href": "http://openstack.example.com:9696/"
                                "%s/ports" % VERSION,
                        "rel": "self"
                    }
                ],
                "name": "port"
            }
        ]
    }

    def setUp(self):
        super(TestNetworkVersionsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.versions_client = versions_client.NetworkVersionsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_versions_client(self, func, body, bytes_body=False, **kwargs):
        self.check_service_client_function(
            func, 'tempest.lib.common.rest_client.RestClient.raw_request',
            body, bytes_body, 200, **kwargs)

    def test_list_versions_client_with_str_body(self):
        self._test_versions_client(self.versions_client.list_versions,
                                   self.FAKE_VERSIONS_INFO)

    def test_list_versions_client_with_bytes_body(self):
        self._test_versions_client(self.versions_client.list_versions,
                                   self.FAKE_VERSIONS_INFO, bytes_body=True)

    def test_show_version_client_with_str_body(self):
        self._test_versions_client(self.versions_client.show_version,
                                   self.FAKE_VERSION_DETAILS,
                                   version=self.VERSION)

    def test_show_version_client_with_bytes_body(self):
        self._test_versions_client(self.versions_client.show_version,
                                   self.FAKE_VERSION_DETAILS, bytes_body=True,
                                   version=self.VERSION)



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures

from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import floating_ips_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestFloatingIpsClient(base.BaseServiceTest):

    floating_ip = {"fixed_ip": None,
                   "id": "46d61064-13ba-4bf0-9557-69de824c3d6f",
                   "instance_id": "a1daa443-a6bb-463e-aea2-104b7d912eb8",
                   "ip": "10.10.10.1",
                   "pool": "nova"}

    def setUp(self):
        super(TestFloatingIpsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = floating_ips_client.FloatingIPsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_floating_ips(self, bytes_body=False):
        expected = {'floating_ips': [TestFloatingIpsClient.floating_ip]}
        self.check_service_client_function(
            self.client.list_floating_ips,
            'tempest.lib.common.rest_client.RestClient.get',
            expected,
            bytes_body)

    def test_list_floating_ips_str_body(self):
        self._test_list_floating_ips(bytes_body=False)

    def test_list_floating_ips_byte_body(self):
        self._test_list_floating_ips(bytes_body=True)

    def _test_show_floating_ip(self, bytes_body=False):
        expected = {"floating_ip": TestFloatingIpsClient.floating_ip}
        self.check_service_client_function(
            self.client.show_floating_ip,
            'tempest.lib.common.rest_client.RestClient.get',
            expected,
            bytes_body,
            floating_ip_id='a1daa443-a6bb-463e-aea2-104b7d912eb8')

    def test_show_floating_ip_str_body(self):
        self._test_show_floating_ip(bytes_body=False)

    def test_show_floating_ip_byte_body(self):
        self._test_show_floating_ip(bytes_body=True)

    def _test_create_floating_ip(self, bytes_body=False):
        expected = {"floating_ip": TestFloatingIpsClient.floating_ip}
        self.check_service_client_function(
            self.client.create_floating_ip,
            'tempest.lib.common.rest_client.RestClient.post',
            expected,
            bytes_body,
            pool_name='nova')

    def test_create_floating_ip_str_body(self):
        self._test_create_floating_ip(bytes_body=False)

    def test_create_floating_ip_byte_body(self):
        self._test_create_floating_ip(bytes_body=True)

    def test_delete_floating_ip(self):
        self.check_service_client_function(
            self.client.delete_floating_ip,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=202, floating_ip_id='fake-id')

    def test_associate_floating_ip_to_server(self):
        self.check_service_client_function(
            self.client.associate_floating_ip_to_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {}, status=202, floating_ip='10.10.10.1',
            server_id='c782b7a9-33cd-45f0-b795-7f87f456408b')

    def test_disassociate_floating_ip_from_server(self):
        self.check_service_client_function(
            self.client.disassociate_floating_ip_from_server,
            'tempest.lib.common.rest_client.RestClient.post',
            {}, status=202, floating_ip='10.10.10.1',
            server_id='c782b7a9-33cd-45f0-b795-7f87f456408b')

    def test_is_resource_deleted_true(self):
        self.useFixture(fixtures.MockPatch(
            'tempest.lib.services.compute.floating_ips_client.'
            'FloatingIPsClient.show_floating_ip',
            side_effect=lib_exc.NotFound()))
        self.assertTrue(self.client.is_resource_deleted('fake-id'))

    def test_is_resource_deleted_false(self):
        self.useFixture(fixtures.MockPatch(
            'tempest.lib.services.compute.floating_ips_client.'
            'FloatingIPsClient.show_floating_ip',
            return_value={"floating_ip": TestFloatingIpsClient.floating_ip}))
        self.assertFalse(self.client.is_resource_deleted('fake-id'))



# Copyright 2013 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 \
    import aggregates as schema
from tempest.lib.api_schema.response.compute.v2_41 \
    import aggregates as schemav241
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import base_compute_client


class AggregatesClient(base_compute_client.BaseComputeClient):

    schema_versions_info = [
        {'min': None, 'max': '2.40', 'schema': schema},
        {'min': '2.41', 'max': None, 'schema': schemav241}]

    def list_aggregates(self):
        """Get aggregate list."""
        resp, body = self.get("os-aggregates")
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.list_aggregates, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_aggregate(self, aggregate_id):
        """Get details of the given aggregate."""
        resp, body = self.get("os-aggregates/%s" % aggregate_id)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.get_aggregate, resp, body)
        return rest_client.ResponseBody(resp, body)

    def create_aggregate(self, **kwargs):
        """Create a new aggregate.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-aggregate
        """
        post_body = json.dumps({'aggregate': kwargs})
        resp, body = self.post('os-aggregates', post_body)

        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.create_aggregate, resp, body)
        return rest_client.ResponseBody(resp, body)

    def update_aggregate(self, aggregate_id, **kwargs):
        """Update an aggregate.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#update-aggregate
        """
        put_body = json.dumps({'aggregate': kwargs})
        resp, body = self.put('os-aggregates/%s' % aggregate_id, put_body)

        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.update_aggregate, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_aggregate(self, aggregate_id):
        """Delete the given aggregate."""
        resp, body = self.delete("os-aggregates/%s" % aggregate_id)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.delete_aggregate, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_aggregate(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Return the primary type of resource this client works with."""
        return 'aggregate'

    def add_host(self, aggregate_id, **kwargs):
        """Add a host to the given aggregate.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#add-host
        """
        post_body = json.dumps({'add_host': kwargs})
        resp, body = self.post('os-aggregates/%s/action' % aggregate_id,
                               post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.aggregate_add_remove_host, resp, body)
        return rest_client.ResponseBody(resp, body)

    def remove_host(self, aggregate_id, **kwargs):
        """Remove a host from the given aggregate.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#remove-host
        """
        post_body = json.dumps({'remove_host': kwargs})
        resp, body = self.post('os-aggregates/%s/action' % aggregate_id,
                               post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.aggregate_add_remove_host, resp, body)
        return rest_client.ResponseBody(resp, body)

    def set_metadata(self, aggregate_id, **kwargs):
        """Replace the aggregate's existing metadata with new metadata.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#create-or-update-aggregate-metadata
        """
        post_body = json.dumps({'set_metadata': kwargs})
        resp, body = self.post('os-aggregates/%s/action' % aggregate_id,
                               post_body)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.aggregate_set_metadata, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2013 Huawei Technologies Co.,LTD.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class HypervisorAdminNegativeTestBase(base.BaseV2ComputeAdminTest):
    """Tests Hypervisors API that require admin privileges"""

    @classmethod
    def setup_clients(cls):
        super(HypervisorAdminNegativeTestBase, cls).setup_clients()
        cls.client = cls.os_admin.hypervisor_client
        cls.non_adm_client = cls.hypervisor_client

    def _list_hypervisors(self):
        # List of hypervisors
        hypers = self.client.list_hypervisors()['hypervisors']
        return hypers


class HypervisorAdminNegativeTestJSON(HypervisorAdminNegativeTestBase):
    """Tests Hypervisors API that require admin privileges"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c136086a-0f67-4b2b-bc61-8482bd68989f')
    def test_show_nonexistent_hypervisor(self):
        """Test showing non existent hypervisor should fail"""
        nonexistent_hypervisor_id = data_utils.rand_uuid()

        self.assertRaises(
            lib_exc.NotFound,
            self.client.show_hypervisor,
            nonexistent_hypervisor_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('51e663d0-6b89-4817-a465-20aca0667d03')
    def test_show_hypervisor_with_non_admin_user(self):
        """Test showing hypervisor by non admin user should fail"""
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers)

        self.assertRaises(
            lib_exc.Forbidden,
            self.non_adm_client.show_hypervisor,
            hypers[0]['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e2b061bb-13f9-40d8-9d6e-d5bf17595849')
    def test_get_hypervisor_stats_with_non_admin_user(self):
        """Test getting hypervisor stats by non admin user should fail"""
        self.assertRaises(
            lib_exc.Forbidden,
            self.non_adm_client.show_hypervisor_statistics)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f60aa680-9a3a-4c7d-90e1-fae3a4891303')
    def test_get_nonexistent_hypervisor_uptime(self):
        """Test showing uptime of non existent hypervisor should fail"""
        nonexistent_hypervisor_id = data_utils.rand_uuid()

        self.assertRaises(
            lib_exc.NotFound,
            self.client.show_hypervisor_uptime,
            nonexistent_hypervisor_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6c3461f9-c04c-4e2a-bebb-71dc9cb47df2')
    def test_get_hypervisor_uptime_with_non_admin_user(self):
        """Test showing uptime of hypervisor by non admin user should fail"""
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers)

        self.assertRaises(
            lib_exc.Forbidden,
            self.non_adm_client.show_hypervisor_uptime,
            hypers[0]['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('51b3d536-9b14-409c-9bce-c6f7c794994e')
    def test_get_hypervisor_list_with_non_admin_user(self):
        """Test listing hypervisors by non admin user should fail"""
        self.assertRaises(
            lib_exc.Forbidden,
            self.non_adm_client.list_hypervisors)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('dc02db05-e801-4c5f-bc8e-d915290ab345')
    def test_get_hypervisor_list_details_with_non_admin_user(self):
        """Test listing hypervisor details by non admin user should fail"""
        self.assertRaises(
            lib_exc.Forbidden,
            self.non_adm_client.list_hypervisors, detail=True)


class HypervisorAdminNegativeUnderV252Test(HypervisorAdminNegativeTestBase):
    """Tests Hypervisors API below ver 2.53 that require admin privileges"""

    max_microversion = '2.52'

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('2a0a3938-832e-4859-95bf-1c57c236b924')
    def test_show_servers_with_non_admin_user(self):
        """Test showing hypervisor servers by non admin user should fail"""
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers)

        self.assertRaises(
            lib_exc.Forbidden,
            self.non_adm_client.list_servers_on_hypervisor,
            hypers[0]['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('02463d69-0ace-4d33-a4a8-93d7883a2bba')
    def test_show_servers_with_nonexistent_hypervisor(self):
        """Test showing servers on non existent hypervisor should fail"""
        nonexistent_hypervisor_id = data_utils.rand_uuid()

        self.assertRaises(
            lib_exc.NotFound,
            self.client.list_servers_on_hypervisor,
            nonexistent_hypervisor_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('5b6a6c79-5dc1-4fa5-9c58-9c8085948e74')
    def test_search_hypervisor_with_non_admin_user(self):
        """Test searching hypervisor by non admin user should fail"""
        hypers = self._list_hypervisors()
        self.assertNotEmpty(hypers)

        self.assertRaises(
            lib_exc.Forbidden,
            self.non_adm_client.search_hypervisor,
            hypers[0]['hypervisor_hostname'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('19a45cc1-1000-4055-b6d2-28e8b2ec4faa')
    def test_search_nonexistent_hypervisor(self):
        """Test searching non existent hypervisor should fail"""
        self.assertRaises(
            lib_exc.NotFound,
            self.client.search_hypervisor,
            'nonexistent_hypervisor_name')



# Copyright 2017 FiberHome Telecommunication Technologies CO.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.volume.v3 import extensions_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestExtensionsClient(base.BaseServiceTest):

    FAKE_EXTENSION_LIST = {
        "extensions": [
            {
                "updated": "2012-03-12T00:00:00+00:00",
                "name": "QuotaClasses",
                "links": [],
                "namespace": "fake-namespace-1",
                "alias": "os-quota-class-sets",
                "description": "Quota classes management support."
            },
            {
                "updated": "2013-05-29T00:00:00+00:00",
                "name": "VolumeTransfer",
                "links": [],
                "namespace": "fake-namespace-2",
                "alias": "os-volume-transfer",
                "description": "Volume transfer management support."
            },
            {
                "updated": "2014-02-10T00:00:00+00:00",
                "name": "VolumeManage",
                "links": [],
                "namespace": "fake-namespace-3",
                "alias": "os-volume-manage",
                "description": "Manage existing backend storage by Cinder."
            }
        ]
    }

    def setUp(self):
        super(TestExtensionsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = extensions_client.ExtensionsClient(fake_auth,
                                                         'volume',
                                                         'regionOne')

    def _test_list_extensions(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_extensions,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_EXTENSION_LIST,
            bytes_body)

    def test_list_extensions_with_str_body(self):
        self._test_list_extensions()

    def test_list_extensions_with_bytes_body(self):
        self._test_list_extensions(bytes_body=True)



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import capabilities as schema
from tempest.lib.common import rest_client


class CapabilitiesClient(rest_client.RestClient):

    def show_backend_capabilities(self, host):
        """Shows capabilities for a storage back end.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v2/index.html#show-back-end-capabilities
        """
        url = 'capabilities/%s' % host
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_backend_capabilities, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import binascii
import hashlib
import hmac
import random
import time
from urllib import parse as urlparse

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class OAUTHTokenClient(rest_client.RestClient):
    api_version = "v3"

    def _escape(self, s):
        """Escape a unicode string in an OAuth-compatible fashion."""
        safe = b'~'
        s = s.encode('utf-8') if isinstance(s, str) else s
        s = urlparse.quote(s, safe)
        if isinstance(s, bytes):
            s = s.decode('utf-8')
        return s

    def _generate_params_with_signature(self, client_key, uri,
                                        client_secret=None,
                                        resource_owner_key=None,
                                        resource_owner_secret=None,
                                        callback_uri=None,
                                        verifier=None,
                                        http_method='GET'):
        """Generate OAUTH params along with signature."""
        timestamp = str(int(time.time()))
        nonce = str(random.getrandbits(64)) + timestamp
        oauth_params = [
            ('oauth_nonce', nonce),
            ('oauth_timestamp', timestamp),
            ('oauth_version', '1.0'),
            ('oauth_signature_method', 'HMAC-SHA1'),
            ('oauth_consumer_key', client_key),
        ]
        if resource_owner_key:
            oauth_params.append(('oauth_token', resource_owner_key))
        if callback_uri:
            oauth_params.append(('oauth_callback', callback_uri))
        if verifier:
            oauth_params.append(('oauth_verifier', verifier))

        # normalize_params
        key_values = [(self._escape(k), self._escape(v))
                      for k, v in oauth_params]
        key_values.sort()
        parameter_parts = ['{0}={1}'.format(k, v) for k, v in key_values]
        normalized_params = '&'.join(parameter_parts)

        # normalize_uri
        scheme, netloc, path, params, _, _ = urlparse.urlparse(uri)
        scheme = scheme.lower()
        netloc = netloc.lower()
        path = path.replace('//', '/')
        normalized_uri = urlparse.urlunparse((scheme, netloc, path,
                                              params, '', ''))

        # construct base string
        base_string = self._escape(http_method.upper())
        base_string += '&'
        base_string += self._escape(normalized_uri)
        base_string += '&'
        base_string += self._escape(normalized_params)

        # sign using hmac-sha1
        key = self._escape(client_secret or '')
        key += '&'
        key += self._escape(resource_owner_secret or '')
        key_utf8 = key.encode('utf-8')
        text_utf8 = base_string.encode('utf-8')
        signature = hmac.new(key_utf8, text_utf8, hashlib.sha1)
        sig = binascii.b2a_base64(signature.digest())[:-1].decode('utf-8')

        oauth_params.append(('oauth_signature', sig))
        return oauth_params

    def _generate_oauth_header(self, oauth_params):
        authorization_header = {}
        authorization_header_parameters_parts = []
        for oauth_parameter_name, value in oauth_params:
            escaped_name = self._escape(oauth_parameter_name)
            escaped_value = self._escape(value)
            part = '{0}="{1}"'.format(escaped_name, escaped_value)
            authorization_header_parameters_parts.append(part)

        authorization_header_parameters = ', '.join(
            authorization_header_parameters_parts)
        oauth_string = 'OAuth %s' % authorization_header_parameters
        authorization_header['Authorization'] = oauth_string

        return authorization_header

    def create_request_token(self, consumer_key, consumer_secret, project_id):
        """Create request token.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#create-request-token
        """
        endpoint = 'OS-OAUTH1/request_token'
        headers = {'Requested-Project-Id': project_id}
        oauth_params = self._generate_params_with_signature(
            consumer_key,
            self.base_url + '/' + endpoint,
            client_secret=consumer_secret,
            callback_uri='oob',
            http_method='POST')
        oauth_header = self._generate_oauth_header(oauth_params)
        headers.update(oauth_header)
        resp, body = self.post(endpoint,
                               body=None,
                               headers=headers)
        self.expected_success(201, resp.status)
        if not isinstance(body, str):
            body = body.decode('utf-8')
        body = dict(item.split("=") for item in body.split("&"))
        return rest_client.ResponseBody(resp, body)

    def authorize_request_token(self, request_token_id, role_ids):
        """Authorize request token.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#authorize-request-token
        """
        roles = [{'id': role_id} for role_id in role_ids]
        body = {'roles': roles}
        post_body = json.dumps(body)
        resp, body = self.put("OS-OAUTH1/authorize/%s" % request_token_id,
                              post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_access_token(self, consumer_key, consumer_secret, request_key,
                            request_secret, oauth_verifier):
        """Create access token.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#create-access-token
        """
        endpoint = 'OS-OAUTH1/access_token'
        oauth_params = self._generate_params_with_signature(
            consumer_key,
            self.base_url + '/' + endpoint,
            client_secret=consumer_secret,
            resource_owner_key=request_key,
            resource_owner_secret=request_secret,
            verifier=oauth_verifier,
            http_method='POST')
        headers = self._generate_oauth_header(oauth_params)
        resp, body = self.post(endpoint, body=None, headers=headers)
        self.expected_success(201, resp.status)
        if not isinstance(body, str):
            body = body.decode('utf-8')
        body = dict(item.split("=") for item in body.split("&"))
        return rest_client.ResponseBody(resp, body)

    def get_access_token(self, user_id, access_token_id):
        """Get access token.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#get-access-token
        """
        resp, body = self.get("users/%s/OS-OAUTH1/access_tokens/%s"
                              % (user_id, access_token_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def revoke_access_token(self, user_id, access_token_id):
        """Revoke access token.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#revoke-access-token
        """
        resp, body = self.delete("users/%s/OS-OAUTH1/access_tokens/%s"
                                 % (user_id, access_token_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_access_tokens(self, user_id):
        """List access tokens.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#list-access-tokens
        """
        resp, body = self.get("users/%s/OS-OAUTH1/access_tokens"
                              % (user_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_access_token_roles(self, user_id, access_token_id):
        """List roles for an access token.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#list-roles-for-an-access-token
        """
        resp, body = self.get("users/%s/OS-OAUTH1/access_tokens/%s/roles"
                              % (user_id, access_token_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def get_access_token_role(self, user_id, access_token_id, role_id):
        """Show role details for an access token.

        For more information, please refer to the official API reference:
        https://docs.openstack.org/api-ref/identity/v3-ext/#show-role-details-for-an-access-token
        """
        resp, body = self.get("users/%s/OS-OAUTH1/access_tokens/%s/roles/%s"
                              % (user_id, access_token_id, role_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class FloatingIPNegativeTestJSON(base.BaseNetworkTest):
    """Test the following negative operations for floating ips:

        Create floatingip with a port that is unreachable to external network
        Create floatingip in private network
        Associate floatingip with port that is unreachable to external network
    """

    @classmethod
    def skip_checks(cls):
        super(FloatingIPNegativeTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('router', 'network'):
            msg = "router extension not enabled."
            raise cls.skipException(msg)
        if not CONF.network.public_network_id:
            msg = "The public_network_id option must be specified."
            raise cls.skipException(msg)
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @classmethod
    def resource_setup(cls):
        super(FloatingIPNegativeTestJSON, cls).resource_setup()
        cls.ext_net_id = CONF.network.public_network_id
        # Create a network with a subnet connected to a router.
        cls.network = cls.create_network()
        subnet = cls.create_subnet(cls.network)
        router = cls.create_router()
        cls.create_router_interface(router['id'], subnet['id'])
        cls.port = cls.create_port(cls.network)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('22996ea8-4a81-4b27-b6e1-fa5df92fa5e8')
    def test_create_floatingip_with_port_ext_net_unreachable(self):
        """Creating floating ip when port's external network is unreachable"""
        self.assertRaises(
            lib_exc.NotFound, self.floating_ips_client.create_floatingip,
            floating_network_id=self.ext_net_id, port_id=self.port['id'],
            fixed_ip_address=self.port['fixed_ips'][0]
                                      ['ip_address'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('50b9aeb4-9f0b-48ee-aa31-fa955a48ff54')
    def test_create_floatingip_in_private_network(self):
        """Test creating floating in private network"""
        self.assertRaises(lib_exc.BadRequest,
                          self.floating_ips_client.create_floatingip,
                          floating_network_id=self.network['id'],
                          port_id=self.port['id'],
                          fixed_ip_address=self.port['fixed_ips'][0]
                                                    ['ip_address'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6b3b8797-6d43-4191-985c-c48b773eb429')
    def test_associate_floatingip_port_ext_net_unreachable(self):
        """Associate floating ip to port with unreachable external network"""
        # Create floating ip
        body = self.floating_ips_client.create_floatingip(
            floating_network_id=self.ext_net_id)
        floating_ip = body['floatingip']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.floating_ips_client.delete_floatingip, floating_ip['id'])
        # Associate floating IP to the other port
        self.assertRaises(
            lib_exc.NotFound, self.floating_ips_client.update_floatingip,
            floating_ip['id'], port_id=self.port['id'],
            fixed_ip_address=self.port['fixed_ips'][0]['ip_address'])



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
# Copyright 2013 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from tempest.api.image import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ImagesNegativeTest(base.BaseV2ImageTest):

    """here we have -ve tests for show_image and delete_image api

    Tests
        ** get non-existent image
        ** get image with image_id=NULL
        ** get the deleted image
        ** delete non-existent image
        ** delete image with image_id=NULL
        ** delete the deleted image
     """

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('668743d5-08ad-4480-b2b8-15da34f81d9f')
    def test_get_non_existent_image(self):
        """Get the non-existent image"""
        non_existent_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.show_image,
                          non_existent_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ef45000d-0a72-4781-866d-4cb7bf2562ad')
    def test_get_image_null_id(self):
        """Get image with image_id = NULL"""
        image_id = ""
        self.assertRaises(lib_exc.NotFound, self.client.show_image, image_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e57fc127-7ba0-4693-92d7-1d8a05ebcba9')
    def test_get_delete_deleted_image(self):
        """Get and delete the deleted image"""
        # create and delete image
        image_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="test")
        image = self.client.create_image(name=image_name,
                                         container_format='bare',
                                         disk_format='raw')
        self.client.delete_image(image['id'])
        self.client.wait_for_resource_deletion(image['id'])

        # get the deleted image
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_image, image['id'])

        # delete the deleted image
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          image['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6fe40f1c-57bd-4918-89cc-8500f850f3de')
    def test_delete_non_existing_image(self):
        """Delete non-existent image"""
        non_existent_image_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          non_existent_image_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('32248db1-ab88-4821-9604-c7c369f1f88c')
    def test_delete_image_null_id(self):
        """Delete image with image_id=NULL"""
        image_id = ""
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          image_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('292bd310-369b-41c7-a7a3-10276ef76753')
    def test_register_with_invalid_container_format(self):
        """Create image with invalid container format

        Negative tests for invalid data supplied to POST /images
        """
        self.assertRaises(lib_exc.BadRequest, self.client.create_image,
                          name='test', container_format='wrong',
                          disk_format='vhd')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('70c6040c-5a97-4111-9e13-e73665264ce1')
    def test_register_with_invalid_disk_format(self):
        """Create image with invalid disk format"""
        self.assertRaises(lib_exc.BadRequest, self.client.create_image,
                          name='test', container_format='bare',
                          disk_format='wrong')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ab980a34-8410-40eb-872b-f264752f46e5')
    def test_delete_protected_image(self):
        """Create a protected image"""
        image_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="test")
        image = self.create_image(name=image_name, protected=True)
        self.addCleanup(self.client.update_image, image['id'],
                        [dict(replace="/protected", value=False)])

        # Try deleting the protected image
        self.assertRaises(lib_exc.Forbidden,
                          self.client.delete_image,
                          image['id'])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a0ae75d4-ce9c-4576-b823-aba04c8acabd')
    def test_update_image_reserved_property(self):
        """Attempt to add a reserved property to an image.

        Glance bans some internal-use-only properties such that they will
        cause a PATCH to fail. Since os_glance_* is banned, we can use a
        key in that namespace here.
        """
        if not CONF.image_feature_enabled.os_glance_reserved:
            raise self.skipException('os_glance_reserved is not enabled')

        image_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="test")
        image = self.create_image(name=image_name,
                                  container_format='bare',
                                  disk_format='raw')
        self.assertRaises(lib_exc.Forbidden,
                          self.client.update_image,
                          image['id'], [{'add': '/os_glance_foo',
                                         'value': 'bar'}])

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e3fb7df8-2742-4143-8976-f1b26870f0a0')
    def test_create_image_reserved_property(self):
        """Attempt to create an image with a reserved property.

        Glance bans some internal-use-only properties such that they will
        cause an image create to fail. Since os_glance_* is banned, we can
        use a key in that namespace here.
        """
        if not CONF.image_feature_enabled.os_glance_reserved:
            raise self.skipException('os_glance_reserved is not enabled')

        image_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="test")
        self.assertRaises(lib_exc.Forbidden,
                          self.create_image,
                          name=image_name,
                          container_format='bare',
                          disk_format='raw',
                          os_glance_foo='bar')


class ImportImagesNegativeTest(base.BaseV2ImageTest):
    """Here we test the import operations for image"""

    @classmethod
    def skip_checks(cls):
        super(ImportImagesNegativeTest, cls).skip_checks()
        if not CONF.image_feature_enabled.import_image:
            skip_msg = (
                "%s skipped as image import is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(ImportImagesNegativeTest, cls).resource_setup()
        cls.available_import_methods = cls.client.info_import()[
            'import-methods']['value']
        if not cls.available_import_methods:
            raise cls.skipException('Server does not support '
                                    'any import method')

        cls.available_stores = cls.get_available_stores()
        if not len(cls.available_stores) > 0:
            raise cls.skipException(
                'No stores configured %s' % cls.available_stores)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('c52f6a77-f522-4411-8dbe-9d14f2b1ba6b')
    def test_image_web_download_import_with_bad_url(self):
        """Test 'web-download' import functionalities

        Make sure that web-download with invalid URL fails properly.
        """
        if 'web-download' not in self.available_import_methods:
            raise self.skipException('Server does not support '
                                     'web-download import method')
        image_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name="test")
        image = self.client.create_image(name=image_name,
                                         container_format='bare',
                                         disk_format='raw')
        # Now try to get image details
        body = self.client.show_image(image['id'])
        self.assertEqual(image['id'], body['id'])
        self.assertEqual('queued', body['status'])
        stores = self.get_available_stores()
        # import image from web to backend
        image_uri = 'http://does-not.exist/no/possible/way'
        self.client.image_import(image['id'], method='web-download',
                                 import_params={'uri': image_uri},
                                 stores=[stores[0]['id']])

        start_time = int(time.time())
        while int(time.time()) - start_time < self.client.build_timeout:
            body = self.client.show_image(image['id'])
            if body.get('os_glance_failed_import'):
                # Store ended up in failed list, which is good
                return
            time.sleep(self.client.build_interval)

        self.fail('Image never reported failed store')



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class DefaultDomainTestJSON(base.BaseIdentityV3Test):
    """Test identity default domains"""

    @classmethod
    def setup_clients(cls):
        super(DefaultDomainTestJSON, cls).setup_clients()
        cls.domains_client = cls.os_primary.domains_client

    @classmethod
    def resource_setup(cls):
        super(DefaultDomainTestJSON, cls).resource_setup()
        cls.domain_id = CONF.identity.default_domain_id

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('17a5de24-e6a0-4e4a-a9ee-d85b6e5612b5')
    def test_default_domain_exists(self):
        """Test showing default domain"""
        domain = self.domains_client.show_domain(self.domain_id)['domain']
        self.assertTrue(domain['enabled'])



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from testtools import matchers

from tempest.api.volume import base
from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class VolumesGetTest(base.BaseVolumeTest):
    """Test getting volume info"""

    def _volume_create_get_update_delete(self, **kwargs):
        # Create a volume, Get it's details and Delete the volume
        v_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-Volume')
        metadata = {'Type': 'Test'}
        # Create a volume
        kwargs['name'] = v_name
        kwargs['metadata'] = metadata
        volume = self.volumes_client.create_volume(**kwargs)['volume']
        self.addCleanup(self.delete_volume, self.volumes_client, volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        self.assertEqual(volume['name'], v_name,
                         "The created volume name is not equal "
                         "to the requested name")

        # Get Volume information
        fetched_volume = self.volumes_client.show_volume(
            volume['id'])['volume']
        self.assertEqual(v_name,
                         fetched_volume['name'],
                         'The fetched Volume name is different '
                         'from the created Volume')
        self.assertEqual(volume['id'],
                         fetched_volume['id'],
                         'The fetched Volume id is different '
                         'from the created Volume')
        self.assertThat(fetched_volume['metadata'].items(),
                        matchers.ContainsAll(metadata.items()),
                        'The fetched Volume metadata misses data '
                        'from the created Volume')

        if 'imageRef' in kwargs:
            self.assertEqual('true', fetched_volume['bootable'])
        else:
            self.assertEqual('false', fetched_volume['bootable'])

        # Update Volume
        # Test volume update when display_name is same with original value
        params = {'name': v_name}
        self.volumes_client.update_volume(volume['id'], **params)
        # Test volume update when display_name is new
        new_v_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + '-new-Volume')
        new_desc = 'This is the new description of volume'
        params = {'name': new_v_name,
                  'description': new_desc}
        update_volume = self.volumes_client.update_volume(
            volume['id'], **params)['volume']
        # Assert response body for update_volume method
        self.assertEqual(new_v_name, update_volume['name'])
        self.assertEqual(new_desc, update_volume['description'])
        # Assert response body for show_volume method
        updated_volume = self.volumes_client.show_volume(
            volume['id'])['volume']
        self.assertEqual(volume['id'], updated_volume['id'])
        self.assertEqual(new_v_name, updated_volume['name'])
        self.assertEqual(new_desc, updated_volume['description'])
        self.assertThat(updated_volume['metadata'].items(),
                        matchers.ContainsAll(metadata.items()),
                        'The fetched Volume metadata misses data '
                        'from the created Volume')

        # Test volume create when display_name is none and display_description
        # contains specific characters,
        # then test volume update if display_name is duplicated
        new_v_desc = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name='@#$%^* description')
        params = {'description': new_v_desc,
                  'availability_zone': volume['availability_zone'],
                  'size': CONF.volume.volume_size}
        new_volume = self.volumes_client.create_volume(**params)['volume']
        self.addCleanup(self.delete_volume, self.volumes_client,
                        new_volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                new_volume['id'], 'available')

        params = {'name': volume['name'],
                  'description': volume['description']}
        self.volumes_client.update_volume(new_volume['id'], **params)

        if 'imageRef' in kwargs:
            self.assertEqual('true', updated_volume['bootable'])
        else:
            self.assertEqual('false', updated_volume['bootable'])

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('27fb0e9f-fb64-41dd-8bdb-1ffa762f0d51')
    def test_volume_create_get_update_delete(self):
        """Test Create/Get/Update/Delete of a blank volume"""
        self._volume_create_get_update_delete(size=CONF.volume.volume_size)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('54a01030-c7fc-447c-86ee-c1182beae638')
    @utils.services('image')
    def test_volume_create_get_update_delete_from_image(self):
        """Test Create/Get/Update/Delete of a volume created from image"""
        image = self.images_client.show_image(CONF.compute.image_ref)
        min_disk = image['min_disk']
        disk_size = max(min_disk, CONF.volume.volume_size)
        self._volume_create_get_update_delete(
            imageRef=CONF.compute.image_ref, size=disk_size)

    @decorators.idempotent_id('3f591b4a-7dc6-444c-bd51-77469506b3a1')
    @testtools.skipUnless(CONF.volume_feature_enabled.clone,
                          'Cinder volume clones are disabled')
    def test_volume_create_get_update_delete_as_clone(self):
        """Test Create/Get/Update/Delete of a cloned volume"""
        origin = self.create_volume()
        self._volume_create_get_update_delete(source_volid=origin['id'],
                                              size=CONF.volume.volume_size)


class VolumesSummaryTest(base.BaseVolumeTest):
    """Test volume summary"""

    volume_min_microversion = '3.12'
    volume_max_microversion = 'latest'

    @decorators.idempotent_id('c4f2431e-4920-4736-9e00-4040386b6feb')
    def test_show_volume_summary(self):
        """Test showing volume summary"""
        # check response schema
        self.volumes_client.show_volume_summary()



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import aggregates

# 'uuid' of an aggregate is returned in microversion 2.41
aggregate_for_create = copy.deepcopy(aggregates.aggregate_for_create)
aggregate_for_create['properties'].update({'uuid': {'type': 'string',
                                                    'format': 'uuid'}})
aggregate_for_create['required'].append('uuid')

common_aggregate_info = copy.deepcopy(aggregates.common_aggregate_info)
common_aggregate_info['properties'].update({'uuid': {'type': 'string',
                                                     'format': 'uuid'}})
common_aggregate_info['required'].append('uuid')

list_aggregates = copy.deepcopy(aggregates.list_aggregates)
list_aggregates['response_body']['properties']['aggregates'].update(
    {'items': common_aggregate_info})

get_aggregate = copy.deepcopy(aggregates.get_aggregate)
get_aggregate['response_body']['properties'].update(
    {'aggregate': common_aggregate_info})

aggregate_set_metadata = get_aggregate

update_aggregate = copy.deepcopy(aggregates.update_aggregate)
update_aggregate['response_body']['properties'].update(
    {'aggregate': common_aggregate_info})

create_aggregate = copy.deepcopy(aggregates.create_aggregate)
create_aggregate['response_body']['properties'].update(
    {'aggregate': aggregate_for_create})

aggregate_add_remove_host = get_aggregate

# NOTE(zhufl): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.1 ***
delete_aggregate = copy.deepcopy(aggregates.delete_aggregate)



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types
from tempest.lib.api_schema.response.compute.v2_28 \
    import hypervisors as hypervisorsv228

###########################################################################
#
# 2.33:
#
# hypervisor_links parameter is added to the response body for the following
# APIs:
#
# - GET /os-hypervisors
# - GET /os-hypervisors/detail
###########################################################################
list_search_hypervisors = copy.deepcopy(
    hypervisorsv228.list_search_hypervisors)
list_search_hypervisors['response_body']['properties'].update(
    {'hypervisors_links': parameter_types.links}
)

list_hypervisors_detail = copy.deepcopy(
    hypervisorsv228.list_hypervisors_detail)
list_hypervisors_detail['response_body']['properties'].update(
    {'hypervisors_links': parameter_types.links}
)

# NOTE(zhufl): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.28 ***
get_hypervisor = copy.deepcopy(hypervisorsv228.get_hypervisor)
hypervisor_detail = copy.deepcopy(hypervisorsv228.hypervisor_detail)
get_hypervisor_statistics = \
    copy.deepcopy(hypervisorsv228.get_hypervisor_statistics)
get_hypervisor_uptime = copy.deepcopy(hypervisorsv228.get_hypervisor_uptime)
get_hypervisors_servers = copy.deepcopy(
    hypervisorsv228.get_hypervisors_servers)



# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class TenantsClient(rest_client.RestClient):
    api_version = "v2.0"

    def create_tenant(self, **kwargs):
        """Create a tenant

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-admin/index.html#create-tenant
        """
        post_body = json.dumps({'tenant': kwargs})
        resp, body = self.post('tenants', post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def delete_tenant(self, tenant_id):
        """Delete a tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-admin/index.html#delete-tenant
        """
        resp, body = self.delete('tenants/%s' % str(tenant_id))
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_tenant(self, tenant_id):
        """Get tenant details.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-admin/index.html#show-tenant-details-by-id
        """
        resp, body = self.get('tenants/%s' % str(tenant_id))
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_tenants(self, **params):
        """Returns tenants.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-admin/index.html#list-tenants-admin-endpoint
        """
        url = 'tenants'
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_tenant(self, tenant_id, **kwargs):
        """Updates a tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-admin/index.html#update-tenant
        """
        if 'id' not in kwargs:
            kwargs['id'] = tenant_id
        post_body = json.dumps({'tenant': kwargs})
        resp, body = self.post('tenants/%s' % tenant_id, post_body)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_tenant_users(self, tenant_id, **params):
        """List users for a Tenant.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/identity/v2-admin/index.html#list-users-on-a-tenant
        """
        url = '/tenants/%s/users' % tenant_id
        if params:
            url += '?%s' % urllib.urlencode(params)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import messages as schema
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc
from tempest.lib.services.volume import base_client


class MessagesClient(base_client.BaseClient):
    """Client class to send user messages API requests."""

    def show_message(self, message_id):
        """Show details for a single message."""
        url = 'messages/%s' % str(message_id)
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.show_message, resp, body)
        return rest_client.ResponseBody(resp, body)

    def list_messages(self):
        """List all messages."""
        url = 'messages'
        resp, body = self.get(url)
        body = json.loads(body)
        self.validate_response(schema.list_messages, resp, body)
        return rest_client.ResponseBody(resp, body)

    def delete_message(self, message_id):
        """Delete a single message."""
        url = 'messages/%s' % str(message_id)
        resp, body = self.delete(url)
        self.validate_response(schema.delete_message, resp, body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_message(id)
        except lib_exc.NotFound:
            return True
        return False

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'message'



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

CONF = config.CONF


class PortsAdminExtendedAttrsTestJSON(base.BaseAdminNetworkTest):
    """Test extended attributes of ports"""

    @classmethod
    def setup_clients(cls):
        super(PortsAdminExtendedAttrsTestJSON, cls).setup_clients()
        cls.hyper_client = cls.os_admin.hypervisor_client

    @classmethod
    def resource_setup(cls):
        super(PortsAdminExtendedAttrsTestJSON, cls).resource_setup()
        cls.network = cls.create_network()
        if CONF.service_available.nova:
            hyper_list = cls.hyper_client.list_hypervisors()
            cls.host_id = hyper_list['hypervisors'][0]['hypervisor_hostname']

    @decorators.idempotent_id('8e8569c1-9ac7-44db-8bc1-f5fb2814f29b')
    @utils.services('compute')
    def test_create_port_binding_ext_attr(self):
        """Test creating port with extended attribute"""
        post_body = {"network_id": self.network['id'],
                     "binding:host_id": self.host_id,
                     "name": data_utils.rand_name(
            self.__class__.__name__,
            prefix=CONF.resource_name_prefix)}
        body = self.admin_ports_client.create_port(**post_body)
        port = body['port']
        self.addCleanup(self.admin_ports_client.wait_for_resource_deletion,
                        port['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.admin_ports_client.delete_port, port['id'])
        host_id = port['binding:host_id']
        self.assertIsNotNone(host_id)
        self.assertEqual(self.host_id, host_id)

    @decorators.idempotent_id('6f6c412c-711f-444d-8502-0ac30fbf5dd5')
    @utils.services('compute')
    def test_update_port_binding_ext_attr(self):
        """Test updating port's extended attribute"""
        post_body = {"network_id": self.network['id'],
                     "name": data_utils.rand_name(
            self.__class__.__name__,
            prefix=CONF.resource_name_prefix)}
        body = self.admin_ports_client.create_port(**post_body)
        port = body['port']
        self.addCleanup(self.admin_ports_client.wait_for_resource_deletion,
                        port['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.admin_ports_client.delete_port, port['id'])
        update_body = {"binding:host_id": self.host_id}
        body = self.admin_ports_client.update_port(port['id'], **update_body)
        updated_port = body['port']
        host_id = updated_port['binding:host_id']
        self.assertIsNotNone(host_id)
        self.assertEqual(self.host_id, host_id)

    @decorators.idempotent_id('1c82a44a-6c6e-48ff-89e1-abe7eaf8f9f8')
    @utils.services('compute')
    def test_list_ports_binding_ext_attr(self):
        """Test updating and listing port's extended attribute"""
        # Create a new port
        post_body = {"network_id": self.network['id'],
                     "name": data_utils.rand_name(
            self.__class__.__name__,
            prefix=CONF.resource_name_prefix)}
        body = self.admin_ports_client.create_port(**post_body)
        port = body['port']
        self.addCleanup(self.admin_ports_client.wait_for_resource_deletion,
                        port['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.admin_ports_client.delete_port, port['id'])

        # Update the port's binding attributes so that is now 'bound'
        # to a host
        update_body = {"binding:host_id": self.host_id}
        self.admin_ports_client.update_port(port['id'], **update_body)

        # List all ports, ensure new port is part of list and its binding
        # attributes are set and accurate
        body = self.admin_ports_client.list_ports()
        ports_list = body['ports']
        pids_list = [p['id'] for p in ports_list]
        self.assertIn(port['id'], pids_list)
        listed_port = [p for p in ports_list if p['id'] == port['id']]
        self.assertEqual(1, len(listed_port),
                         'Multiple ports listed with id %s in ports listing: '
                         '%s' % (port['id'], ports_list))
        self.assertEqual(self.host_id, listed_port[0]['binding:host_id'])

    @decorators.idempotent_id('b54ac0ff-35fc-4c79-9ca3-c7dbd4ea4f13')
    def test_show_port_binding_ext_attr(self):
        """Test showing port's extended attribute"""
        body = self.admin_ports_client.create_port(
            name=data_utils.rand_name(
                self.__class__.__name__, prefix=CONF.resource_name_prefix),
            network_id=self.network['id'])
        port = body['port']
        self.addCleanup(self.admin_ports_client.wait_for_resource_deletion,
                        port['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.admin_ports_client.delete_port, port['id'])
        body = self.admin_ports_client.show_port(port['id'])
        show_port = body['port']
        self.assertEqual(port['binding:host_id'],
                         show_port['binding:host_id'])
        self.assertEqual(port['binding:vif_type'],
                         show_port['binding:vif_type'])
        self.assertEqual(port['binding:vif_details'],
                         show_port['binding:vif_details'])


class PortsAdminExtendedAttrsIpV6TestJSON(PortsAdminExtendedAttrsTestJSON):
    _ip_version = 6



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.api.compute import base
from tempest.common import compute
from tempest import config
from tempest import exceptions
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


@decorators.serial
class ServersAffinityTest(base.BaseV2ComputeAdminTest):
    """Test creating servers without multi-create with scheduler_hints.

    The server affinity tests in ServersOnMultiNodesTest use the /servers
    multi-create API and therefore do not test affinity behavior when server
    group members already exist on hosts.

    These tests must be run in serial because they will be disabling compute
    hosts in order to verify affinity behavior.
    """
    # 2.64 added 'policy' and 'rules' fields to POST /os-server-groups
    min_microversion = '2.64'

    @classmethod
    def resource_setup(cls):
        super().resource_setup()

        # Disable all compute hosts except two.
        services = cls.os_admin.services_client.list_services(
            binary='nova-compute')['services']
        num_extra = len(services) - 2
        for i in range(num_extra):
            service = services.pop()
            cls.os_admin.services_client.update_service(
                service['id'], status='disabled')
        cls.services = {
            service['host']: service['id'] for service in services}

    @classmethod
    def skip_checks(cls):
        super().skip_checks()

        if CONF.compute.min_compute_nodes < 2:
            raise cls.skipException(
                "Less than 2 compute nodes, skipping affinity tests.")

    def _disable_compute_host(self, hostname):
        service_id = self.services[hostname]
        self.os_admin.services_client.update_service(
            service_id, status='disabled')
        self.addCleanup(
            self.os_admin.services_client.update_service, service_id,
            status='enabled')

    def _create_server(self, **kwargs):
        body, servers = compute.create_test_server(
            self.os_primary, networks='none', **kwargs)
        for server in servers:
            self.addCleanup(self.delete_server, server['id'])
        return body

    def _create_server_group(self, **kwargs):
        name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=self.__class__.__name__ + "-Server-Group")
        group_id = self.server_groups_client.create_server_group(
            name=name, **kwargs)['server_group']['id']
        self.addCleanup(
            self.server_groups_client.delete_server_group, group_id)
        return group_id

    def _create_server_in_group(self, group_id):
        hints = {'group': group_id}
        server = self._create_server(
            scheduler_hints=hints, wait_until='ACTIVE')
        # Assert the server is in the group.
        server_group = self.server_groups_client.show_server_group(
            group_id)['server_group']
        self.assertIn(server['id'], server_group['members'])
        return server

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('28ef4c29-09db-40a8-aacd-dc5fa321f35e')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAffinityFilter"),
        'ServerGroupAffinityFilter is not available.')
    def test_create_server_with_affinity(self):
        group_id = self._create_server_group(policy='affinity')

        # Create server1 in the group.
        server1 = self._create_server_in_group(group_id)

        # Create server2 in the group.
        server2 = self._create_server_in_group(group_id)

        # Servers should be on the same host.
        self.assertEqual(
            self.get_host_for_server(server1['id']),
            self.get_host_for_server(server2['id']))

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('3ac1ff4e-0fa2-4069-ae59-695cf829275b')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAffinityFilter"),
        'ServerGroupAffinityFilter is not available.')
    def test_create_server_with_affinity_negative(self):
        group_id = self._create_server_group(policy='affinity')

        # Create server1 in the group.
        server1 = self._create_server_in_group(group_id)

        # Disable the compute host server1 is on.
        self._disable_compute_host(self.get_host_for_server(server1['id']))

        # Create server2 in the group. This should fail because affinity policy
        # cannot be honored.
        self.assertRaises(
            exceptions.BuildErrorException,
            self._create_server_in_group, group_id)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('99cf4819-479c-4176-a9a6-ad501f6fc4b7')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAffinityFilter"),
        'ServerGroupAffinityFilter is not available.')
    def test_create_server_with_soft_affinity(self):
        group_id = self._create_server_group(policy='soft-affinity')

        # Create server1 in the group.
        server1 = self._create_server_in_group(group_id)

        # Disable the compute host server1 is on.
        self._disable_compute_host(self.get_host_for_server(server1['id']))

        # Create server2 in the group. This should succeed because soft
        # affinity is best effort and scheduling should go ahead even if the
        # policy cannot be honored.
        server2 = self._create_server_in_group(group_id)

        # Servers should be on different hosts.
        self.assertNotEqual(
            self.get_host_for_server(server1['id']),
            self.get_host_for_server(server2['id']))

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('475e9db0-5512-41cb-a6b2-4bd6fb3c7603')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAntiAffinityFilter"),
        'ServerGroupAntiAffinityFilter is not available.')
    def test_create_server_with_anti_affinity(self):
        group_id = self._create_server_group(policy='anti-affinity')

        # Create server1 in the group.
        server1 = self._create_server_in_group(group_id)

        # Create server2 in the group.
        server2 = self._create_server_in_group(group_id)

        # Servers should be on different hosts.
        self.assertNotEqual(
            self.get_host_for_server(server1['id']),
            self.get_host_for_server(server2['id']))

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('c5e43585-0fdd-42a9-a525-2b99465c28df')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAntiAffinityFilter"),
        'ServerGroupAntiAffinityFilter is not available.')
    def test_create_server_with_anti_affinity_negative(self):
        group_id = self._create_server_group(policy='anti-affinity')

        # Create server1 in the group.
        server1 = self._create_server_in_group(group_id)

        # Disable the compute host server1 is not on.
        self._disable_compute_host(self.get_host_other_than(server1['id']))

        # Create server2 in the group. This should fail because anti-affinity
        # policy cannot be honored.
        self.assertRaises(
            exceptions.BuildErrorException,
            self._create_server_in_group, group_id)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('88a8c3d4-c0e8-4873-ba6f-006004779f29')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAntiAffinityFilter"),
        'ServerGroupAntiAffinityFilter is not available.')
    def test_create_server_with_anti_affinity_max_server_per_host(self):
        group_id = self._create_server_group(
            policy='anti-affinity', rules={'max_server_per_host': 2})

        # Create server1 in the group.
        server1 = self._create_server_in_group(group_id)

        # Disable the compute host server1 is not on.
        self._disable_compute_host(self.get_host_other_than(server1['id']))

        # Create server2 in the group. This should succeed because we are
        # allowing a maximum of two servers per compute host for anti-affinity.
        server2 = self._create_server_in_group(group_id)

        # Servers should be on the same host.
        self.assertEqual(
            self.get_host_for_server(server1['id']),
            self.get_host_for_server(server2['id']))

        # A attempt to create a third server in the group should fail because
        # we have already reached our maximum allowed servers per compute host
        # for anti-affinity.
        self.assertRaises(
            exceptions.BuildErrorException,
            self._create_server_in_group, group_id)

    @decorators.attr(type='multinode')
    @decorators.idempotent_id('ef2bc189-5ecc-4a23-8c1b-0d70a9138a77')
    @testtools.skipUnless(
        compute.is_scheduler_filter_enabled("ServerGroupAntiAffinityFilter"),
        'ServerGroupAntiAffinityFilter is not available.')
    def test_create_server_with_soft_anti_affinity(self):
        group_id = self._create_server_group(policy='soft-anti-affinity')

        # Create server1 in the group.
        server1 = self._create_server_in_group(group_id)

        # Disable the compute host server1 is not on.
        self._disable_compute_host(self.get_host_other_than(server1['id']))

        # Create server2 in the group. This should succeed because soft
        # anti-affinity is best effort and scheduling should go ahead even if
        # the policy cannot be honored.
        server2 = self._create_server_in_group(group_id)

        # Servers should be on the same host.
        self.assertEqual(
            self.get_host_for_server(server1['id']),
            self.get_host_for_server(server2['id']))



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import fixtures

from tempest.lib import exceptions as lib_exc
from tempest.lib.services.compute import volumes_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestVolumesClient(base.BaseServiceTest):

    FAKE_VOLUME = {
        "id": "521752a6-acf6-4b2d-bc7a-119f9148cd8c",
        "displayName": u"v\u12345ol-001",
        "displayDescription": u"Another \u1234volume.",
        "size": 30,
        "status": "Active",
        "volumeType": "289da7f8-6440-407c-9fb4-7db01ec49164",
        "metadata": {
            "contents": "junk"
        },
        "availabilityZone": "us-east1",
        "snapshotId": None,
        "attachments": [],
        "createdAt": "2012-02-14T20:53:07Z"
    }

    FAKE_VOLUMES = {"volumes": [FAKE_VOLUME]}

    def setUp(self):
        super(TestVolumesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = volumes_client.VolumesClient(
            fake_auth, 'compute', 'regionOne')

    def _test_list_volumes(self, bytes_body=False, **params):
        self.check_service_client_function(
            self.client.list_volumes,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_VOLUMES, to_utf=bytes_body, **params)

    def test_list_volumes_with_str_body(self):
        self._test_list_volumes()

    def test_list_volumes_with_byte_body(self):
        self._test_list_volumes(bytes_body=True)

    def test_list_volumes_with_params(self):
        self._test_list_volumes(name='fake')

    def _test_show_volume(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_volume,
            'tempest.lib.common.rest_client.RestClient.get',
            {"volume": self.FAKE_VOLUME},
            to_utf=bytes_body, volume_id=self.FAKE_VOLUME['id'])

    def test_show_volume_with_str_body(self):
        self._test_show_volume()

    def test_show_volume_with_bytes_body(self):
        self._test_show_volume(bytes_body=True)

    def _test_create_volume(self, bytes_body=False):
        post_body = copy.deepcopy(self.FAKE_VOLUME)
        del post_body['id']
        del post_body['createdAt']
        del post_body['status']
        self.check_service_client_function(
            self.client.create_volume,
            'tempest.lib.common.rest_client.RestClient.post',
            {"volume": self.FAKE_VOLUME},
            to_utf=bytes_body, status=200, **post_body)

    def test_create_volume_with_str_body(self):
        self._test_create_volume()

    def test_create_volume_with_bytes_body(self):
        self._test_create_volume(bytes_body=True)

    def test_delete_volume(self):
        self.check_service_client_function(
            self.client.delete_volume,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=202, volume_id=self.FAKE_VOLUME['id'])

    def test_is_resource_deleted_true(self):
        module = ('tempest.lib.services.compute.volumes_client.'
                  'VolumesClient.show_volume')
        self.useFixture(fixtures.MockPatch(
            module, side_effect=lib_exc.NotFound))
        self.assertTrue(self.client.is_resource_deleted('fake-id'))

    def test_is_resource_deleted_false(self):
        module = ('tempest.lib.services.compute.volumes_client.'
                  'VolumesClient.show_volume')
        self.useFixture(fixtures.MockPatch(
            module, return_value={}))
        self.assertFalse(self.client.is_resource_deleted('fake-id'))



# Copyright 2021 Red Hat, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class FloatingIpsPortForwardingClient(base.BaseNetworkClient):

    def create_port_forwarding(self, floatingip_id, **kwargs):
        """Creates a floating IP port forwarding.

        Creates port forwarding by using the configuration that you define in
        the request object.
        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-port-forwarding
        """
        uri = '/floatingips/%s/port_forwardings' % floatingip_id
        post_data = {'port_forwarding': kwargs}
        return self.create_resource(uri, post_data)

    def update_port_forwarding(
            self, floatingip_id, port_forwarding_id, **kwargs):
        """Updates a floating IP port_forwarding resource.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-a-port-forwarding
        """
        uri = '/floatingips/%s/port_forwardings/%s' % (
            floatingip_id, port_forwarding_id)
        post_data = {'port_forwarding': kwargs}
        return self.update_resource(uri, post_data)

    def show_port_forwarding(
            self, floatingip_id, port_forwarding_id, **fields):
        """Shows details for a floating IP port forwarding id.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-port-forwarding
        """
        uri = '/floatingips/%s/port_forwardings/%s' % (
            floatingip_id, port_forwarding_id)
        return self.show_resource(uri, **fields)

    def delete_port_forwarding(self, floatingip_id, port_forwarding_id):
        """Deletes a floating IP port_forwarding resource.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#delete-a-floating-ip-port-forwarding
        """
        uri = '/floatingips/%s/port_forwardings/%s' % (
            floatingip_id, port_forwarding_id)
        return self.delete_resource(uri)

    def list_port_forwardings(self, floatingip_id, **filters):
        """Lists floating Ip port forwardings.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-floating-ip-port-forwardings-detail
        """
        uri = '/floatingips/%s/port_forwardings' % floatingip_id
        return self.list_resources(uri, **filters)



# Copyright (c) 2017 IBM Corp.
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock

import fixtures
import testtools

from tempest.lib.common import validation_resources as vr
from tempest.lib import exceptions as lib_exc
from tempest.lib.services import clients
from tempest.tests import base
from tempest.tests.lib import fake_credentials
from tempest.tests.lib.services import registry_fixture

FAKE_SECURITY_GROUP = {'security_group': {'id': 'sg_id'}}
FAKE_KEYPAIR = {'keypair': {'name': 'keypair_name'}}
FAKE_FIP_NOVA_NET = {'floating_ip': {'ip': '1.2.3.4', 'id': '1234'}}
FAKE_FIP_NEUTRON = {'floatingip': {'floating_ip_address': '1.2.3.4',
                                   'id': '1234'}}

SERVICES = 'tempest.lib.services'
SG_CLIENT = (SERVICES + '.%s.security_groups_client.SecurityGroupsClient.%s')
SGR_CLIENT = (SERVICES + '.%s.security_group_rules_client.'
              'SecurityGroupRulesClient.create_security_group_rule')
KP_CLIENT = (SERVICES + '.compute.keypairs_client.KeyPairsClient.%s')
FIP_CLIENT = (SERVICES + '.%s.floating_ips_client.FloatingIPsClient.%s')


class TestValidationResources(base.TestCase):

    def setUp(self):
        super(TestValidationResources, self).setUp()
        self.useFixture(registry_fixture.RegistryFixture())
        self.mock_sg_compute = self.useFixture(fixtures.MockPatch(
            SG_CLIENT % ('compute', 'create_security_group'), autospec=True,
            return_value=FAKE_SECURITY_GROUP))
        self.mock_sg_network = self.useFixture(fixtures.MockPatch(
            SG_CLIENT % ('network', 'create_security_group'), autospec=True,
            return_value=FAKE_SECURITY_GROUP))
        self.mock_sgr_compute = self.useFixture(fixtures.MockPatch(
            SGR_CLIENT % 'compute', autospec=True))
        self.mock_sgr_network = self.useFixture(fixtures.MockPatch(
            SGR_CLIENT % 'network', autospec=True))
        self.mock_kp = self.useFixture(fixtures.MockPatch(
            KP_CLIENT % 'create_keypair', autospec=True,
            return_value=FAKE_KEYPAIR))
        self.mock_fip_compute = self.useFixture(fixtures.MockPatch(
            FIP_CLIENT % ('compute', 'create_floating_ip'), autospec=True,
            return_value=FAKE_FIP_NOVA_NET))
        self.mock_fip_network = self.useFixture(fixtures.MockPatch(
            FIP_CLIENT % ('network', 'create_floatingip'), autospec=True,
            return_value=FAKE_FIP_NEUTRON))
        self.os = clients.ServiceClients(
            fake_credentials.FakeKeystoneV3Credentials(), 'fake_uri')

    def test_create_ssh_security_group_nova_net(self):
        expected_sg_id = FAKE_SECURITY_GROUP['security_group']['id']
        sg = vr.create_ssh_security_group(self.os, add_rule=True,
                                          use_neutron=False)
        self.assertEqual(FAKE_SECURITY_GROUP['security_group'], sg)
        # Neutron clients have not been used
        self.assertEqual(self.mock_sg_network.mock.call_count, 0)
        self.assertEqual(self.mock_sgr_network.mock.call_count, 0)
        # Nova-net clients assertions
        self.assertGreater(self.mock_sg_compute.mock.call_count, 0)
        self.assertGreater(self.mock_sgr_compute.mock.call_count, 0)
        for call in self.mock_sgr_compute.mock.call_args_list[1:]:
            self.assertIn(expected_sg_id, call[1].values())

    def test_create_ssh_security_group_neutron(self):
        expected_sg_id = FAKE_SECURITY_GROUP['security_group']['id']
        expected_ethertype = 'fake_ethertype'
        sg = vr.create_ssh_security_group(self.os, add_rule=True,
                                          use_neutron=True,
                                          ethertype=expected_ethertype)
        self.assertEqual(FAKE_SECURITY_GROUP['security_group'], sg)
        # Nova-net clients have not been used
        self.assertEqual(self.mock_sg_compute.mock.call_count, 0)
        self.assertEqual(self.mock_sgr_compute.mock.call_count, 0)
        # Nova-net clients assertions
        self.assertGreater(self.mock_sg_network.mock.call_count, 0)
        self.assertGreater(self.mock_sgr_network.mock.call_count, 0)
        # Check SG ID and ethertype are passed down to rules
        for call in self.mock_sgr_network.mock.call_args_list[1:]:
            self.assertIn(expected_sg_id, call[1].values())
            self.assertIn(expected_ethertype, call[1].values())

    def test_create_ssh_security_no_rules(self):
        sg = vr.create_ssh_security_group(self.os, add_rule=False)
        self.assertEqual(FAKE_SECURITY_GROUP['security_group'], sg)
        # SG Rules clients have not been used
        self.assertEqual(self.mock_sgr_compute.mock.call_count, 0)
        self.assertEqual(self.mock_sgr_network.mock.call_count, 0)

    @mock.patch.object(vr, 'create_ssh_security_group',
                       return_value=FAKE_SECURITY_GROUP['security_group'])
    def test_create_validation_resources_nova_net(self, mock_create_sg):
        expected_floating_network_id = 'my_fni'
        expected_floating_network_name = 'my_fnn'
        resources = vr.create_validation_resources(
            self.os, keypair=True, floating_ip=True, security_group=True,
            security_group_rules=True, ethertype='IPv6', use_neutron=False,
            floating_network_id=expected_floating_network_id,
            floating_network_name=expected_floating_network_name)
        # Keypair calls
        self.assertGreater(self.mock_kp.mock.call_count, 0)
        # Floating IP calls
        self.assertGreater(self.mock_fip_compute.mock.call_count, 0)
        for call in self.mock_fip_compute.mock.call_args_list[1:]:
            self.assertIn(expected_floating_network_name, call[1].values())
            self.assertNotIn(expected_floating_network_id, call[1].values())
        self.assertEqual(self.mock_fip_network.mock.call_count, 0)
        # SG calls
        mock_create_sg.assert_called_once()
        # Resources
        for resource in ['keypair', 'floating_ip', 'security_group']:
            self.assertIn(resource, resources)
        self.assertEqual(FAKE_KEYPAIR['keypair'], resources['keypair'])
        self.assertEqual(FAKE_SECURITY_GROUP['security_group'],
                         resources['security_group'])
        self.assertEqual(FAKE_FIP_NOVA_NET['floating_ip'],
                         resources['floating_ip'])

    @mock.patch.object(vr, 'create_ssh_security_group',
                       return_value=FAKE_SECURITY_GROUP['security_group'])
    def test_create_validation_resources_neutron(self, mock_create_sg):
        expected_floating_network_id = 'my_fni'
        expected_floating_network_name = 'my_fnn'
        resources = vr.create_validation_resources(
            self.os, keypair=True, floating_ip=True, security_group=True,
            security_group_rules=True, ethertype='IPv6', use_neutron=True,
            floating_network_id=expected_floating_network_id,
            floating_network_name=expected_floating_network_name)
        # Keypair calls
        self.assertGreater(self.mock_kp.mock.call_count, 0)
        # Floating IP calls
        self.assertEqual(self.mock_fip_compute.mock.call_count, 0)
        self.assertGreater(self.mock_fip_network.mock.call_count, 0)
        for call in self.mock_fip_compute.mock.call_args_list[1:]:
            self.assertIn(expected_floating_network_id, call[1].values())
            self.assertNotIn(expected_floating_network_name, call[1].values())
        # SG calls
        mock_create_sg.assert_called_once()
        # Resources
        for resource in ['keypair', 'floating_ip', 'security_group']:
            self.assertIn(resource, resources)
        self.assertEqual(FAKE_KEYPAIR['keypair'], resources['keypair'])
        self.assertEqual(FAKE_SECURITY_GROUP['security_group'],
                         resources['security_group'])
        self.assertIn('ip', resources['floating_ip'])
        self.assertEqual(resources['floating_ip']['ip'],
                         FAKE_FIP_NEUTRON['floatingip']['floating_ip_address'])
        self.assertEqual(resources['floating_ip']['id'],
                         FAKE_FIP_NEUTRON['floatingip']['id'])


class TestClearValidationResourcesFixture(base.TestCase):

    def setUp(self):
        super(TestClearValidationResourcesFixture, self).setUp()
        self.useFixture(registry_fixture.RegistryFixture())
        self.mock_sg_compute = self.useFixture(fixtures.MockPatch(
            SG_CLIENT % ('compute', 'delete_security_group'), autospec=True))
        self.mock_sg_network = self.useFixture(fixtures.MockPatch(
            SG_CLIENT % ('network', 'delete_security_group'), autospec=True))
        self.mock_sg_wait_compute = self.useFixture(fixtures.MockPatch(
            SG_CLIENT % ('compute', 'wait_for_resource_deletion'),
            autospec=True))
        self.mock_sg_wait_network = self.useFixture(fixtures.MockPatch(
            SG_CLIENT % ('network', 'wait_for_resource_deletion'),
            autospec=True))
        self.mock_kp = self.useFixture(fixtures.MockPatch(
            KP_CLIENT % 'delete_keypair', autospec=True))
        self.mock_fip_compute = self.useFixture(fixtures.MockPatch(
            FIP_CLIENT % ('compute', 'delete_floating_ip'), autospec=True))
        self.mock_fip_network = self.useFixture(fixtures.MockPatch(
            FIP_CLIENT % ('network', 'delete_floatingip'), autospec=True))
        self.os = clients.ServiceClients(
            fake_credentials.FakeKeystoneV3Credentials(), 'fake_uri')

    def test_clear_validation_resources_nova_net(self):
        vr.clear_validation_resources(
            self.os,
            floating_ip=FAKE_FIP_NOVA_NET['floating_ip'],
            security_group=FAKE_SECURITY_GROUP['security_group'],
            keypair=FAKE_KEYPAIR['keypair'],
            use_neutron=False)
        self.assertGreater(self.mock_kp.mock.call_count, 0)
        for call in self.mock_kp.mock.call_args_list[1:]:
            self.assertIn(FAKE_KEYPAIR['keypair']['name'], call[1].values())
        self.assertGreater(self.mock_sg_compute.mock.call_count, 0)
        for call in self.mock_sg_compute.mock.call_args_list[1:]:
            self.assertIn(FAKE_SECURITY_GROUP['security_group']['id'],
                          call[1].values())
        self.assertGreater(self.mock_sg_wait_compute.mock.call_count, 0)
        for call in self.mock_sg_wait_compute.mock.call_args_list[1:]:
            self.assertIn(FAKE_SECURITY_GROUP['security_group']['id'],
                          call[1].values())
        self.assertEqual(self.mock_sg_network.mock.call_count, 0)
        self.assertEqual(self.mock_sg_wait_network.mock.call_count, 0)
        self.assertGreater(self.mock_fip_compute.mock.call_count, 0)
        for call in self.mock_fip_compute.mock.call_args_list[1:]:
            self.assertIn(FAKE_FIP_NOVA_NET['floating_ip']['id'],
                          call[1].values())
        self.assertEqual(self.mock_fip_network.mock.call_count, 0)

    def test_clear_validation_resources_neutron(self):
        vr.clear_validation_resources(
            self.os,
            floating_ip=FAKE_FIP_NEUTRON['floatingip'],
            security_group=FAKE_SECURITY_GROUP['security_group'],
            keypair=FAKE_KEYPAIR['keypair'],
            use_neutron=True)
        self.assertGreater(self.mock_kp.mock.call_count, 0)
        for call in self.mock_kp.mock.call_args_list[1:]:
            self.assertIn(FAKE_KEYPAIR['keypair']['name'], call[1].values())
        self.assertGreater(self.mock_sg_network.mock.call_count, 0)
        for call in self.mock_sg_network.mock.call_args_list[1:]:
            self.assertIn(FAKE_SECURITY_GROUP['security_group']['id'],
                          call[1].values())
        self.assertGreater(self.mock_sg_wait_network.mock.call_count, 0)
        for call in self.mock_sg_wait_network.mock.call_args_list[1:]:
            self.assertIn(FAKE_SECURITY_GROUP['security_group']['id'],
                          call[1].values())
        self.assertEqual(self.mock_sg_compute.mock.call_count, 0)
        self.assertEqual(self.mock_sg_wait_compute.mock.call_count, 0)
        self.assertGreater(self.mock_fip_network.mock.call_count, 0)
        for call in self.mock_fip_network.mock.call_args_list[1:]:
            self.assertIn(FAKE_FIP_NEUTRON['floatingip']['id'],
                          call[1].values())
        self.assertEqual(self.mock_fip_compute.mock.call_count, 0)

    def test_clear_validation_resources_exceptions(self):
        # Test that even with exceptions all cleanups are invoked and that only
        # the first exception is reported.
        # NOTE(andreaf) There's not way of knowing which exception is going to
        # be raised first unless we enforce which resource is cleared first,
        # which is not really interesting, but also not harmful. keypair first.
        self.mock_kp.mock.side_effect = Exception('keypair exception')
        self.mock_sg_network.mock.side_effect = Exception('sg exception')
        self.mock_fip_network.mock.side_effect = Exception('fip exception')
        with testtools.ExpectedException(Exception, value_re='keypair'):
            vr.clear_validation_resources(
                self.os,
                floating_ip=FAKE_FIP_NEUTRON['floatingip'],
                security_group=FAKE_SECURITY_GROUP['security_group'],
                keypair=FAKE_KEYPAIR['keypair'],
                use_neutron=True)
        # Clients calls are still made, but not the wait call
        self.assertGreater(self.mock_kp.mock.call_count, 0)
        self.assertGreater(self.mock_sg_network.mock.call_count, 0)
        self.assertGreater(self.mock_fip_network.mock.call_count, 0)

    def test_clear_validation_resources_wait_not_found_wait(self):
        # Test that a not found on wait is not an exception
        self.mock_sg_wait_network.mock.side_effect = lib_exc.NotFound('yay')
        vr.clear_validation_resources(
            self.os,
            floating_ip=FAKE_FIP_NEUTRON['floatingip'],
            security_group=FAKE_SECURITY_GROUP['security_group'],
            keypair=FAKE_KEYPAIR['keypair'],
            use_neutron=True)
        # Clients calls are still made, but not the wait call
        self.assertGreater(self.mock_kp.mock.call_count, 0)
        self.assertGreater(self.mock_sg_network.mock.call_count, 0)
        self.assertGreater(self.mock_sg_wait_network.mock.call_count, 0)
        self.assertGreater(self.mock_fip_network.mock.call_count, 0)

    def test_clear_validation_resources_wait_not_found_delete(self):
        # Test that a not found on delete is not an exception
        self.mock_kp.mock.side_effect = lib_exc.NotFound('yay')
        self.mock_sg_network.mock.side_effect = lib_exc.NotFound('yay')
        self.mock_fip_network.mock.side_effect = lib_exc.NotFound('yay')
        vr.clear_validation_resources(
            self.os,
            floating_ip=FAKE_FIP_NEUTRON['floatingip'],
            security_group=FAKE_SECURITY_GROUP['security_group'],
            keypair=FAKE_KEYPAIR['keypair'],
            use_neutron=True)
        # Clients calls are still made, but not the wait call
        self.assertGreater(self.mock_kp.mock.call_count, 0)
        self.assertGreater(self.mock_sg_network.mock.call_count, 0)
        self.assertEqual(self.mock_sg_wait_network.mock.call_count, 0)
        self.assertGreater(self.mock_fip_network.mock.call_count, 0)


class TestValidationResourcesFixture(base.TestCase):

    @mock.patch.object(vr, 'create_validation_resources', autospec=True)
    def test_use_fixture(self, mock_vr):
        exp_vr = dict(keypair='keypair',
                      floating_ip='floating_ip',
                      security_group='security_group')
        mock_vr.return_value = exp_vr
        exp_clients = 'clients'
        exp_parameters = dict(keypair=True, floating_ip=True,
                              security_group=True, security_group_rules=True,
                              ethertype='v6', use_neutron=True,
                              floating_network_id='fnid',
                              floating_network_name='fnname')
        # First mock cleanup
        self.useFixture(fixtures.MockPatchObject(
            vr, 'clear_validation_resources', autospec=True))
        # And then use vr fixture, so when the fixture is cleaned-up, the mock
        # is still there
        vr_fixture = self.useFixture(vr.ValidationResourcesFixture(
            exp_clients, **exp_parameters))
        # Assert vr have been provisioned
        mock_vr.assert_called_once_with(exp_clients, **exp_parameters)
        # Assert vr have been setup in the fixture
        self.assertEqual(exp_vr, vr_fixture.resources)

    @mock.patch.object(vr, 'clear_validation_resources', autospec=True)
    @mock.patch.object(vr, 'create_validation_resources', autospec=True)
    def test_use_fixture_context(self, mock_vr, mock_clear):
        exp_vr = dict(keypair='keypair',
                      floating_ip='floating_ip',
                      security_group='security_group')
        mock_vr.return_value = exp_vr
        exp_clients = 'clients'
        exp_parameters = dict(keypair=True, floating_ip=True,
                              security_group=True, security_group_rules=True,
                              ethertype='v6', use_neutron=True,
                              floating_network_id='fnid',
                              floating_network_name='fnname')
        with vr.ValidationResourcesFixture(exp_clients,
                                           **exp_parameters) as vr_fixture:
            # Assert vr have been provisioned
            mock_vr.assert_called_once_with(exp_clients, **exp_parameters)
            # Assert vr have been setup in the fixture
            self.assertEqual(exp_vr, vr_fixture.resources)
        # After context manager is closed, clear is invoked
        exp_vr['use_neutron'] = exp_parameters['use_neutron']
        mock_clear.assert_called_once_with(exp_clients, **exp_vr)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

common_floating_ip_info = {
    'type': 'object',
    'properties': {
        # NOTE: Now the type of 'id' is integer, but
        # here allows 'string' also because we will be
        # able to change it to 'uuid' in the future.
        'id': {'type': ['integer', 'string']},
        'pool': {'type': ['string', 'null']},
        'instance_id': {'type': ['string', 'null']},
        'ip': parameter_types.ip_address,
        'fixed_ip': parameter_types.ip_address
    },
    'additionalProperties': False,
    'required': ['id', 'pool', 'instance_id',
                 'ip', 'fixed_ip'],

}
list_floating_ips = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'floating_ips': {
                'type': 'array',
                'items': common_floating_ip_info
            },
        },
        'additionalProperties': False,
        'required': ['floating_ips'],
    }
}

create_get_floating_ip = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'floating_ip': common_floating_ip_info
        },
        'additionalProperties': False,
        'required': ['floating_ip'],
    }
}

add_remove_floating_ip = {
    'status_code': [202]
}



# Copyright 2021 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class LimitsClient(rest_client.RestClient):
    api_version = "v3"

    def get_registered_limits(self):
        """Lists all registered limits."""
        resp, body = self.get('registered_limits')
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, json.loads(body))

    def create_limit(self, region_id, service_id, project_id, resource_name,
                     default_limit, description=None, domain_id=None):
        """Creates a limit in keystone."""
        limit = {
            'service_id': service_id,
            'project_id': project_id,
            'resource_name': resource_name,
            'resource_limit': default_limit,
            'region_id': region_id,
            'description': description or '%s limit for %s' % (
                resource_name, project_id),
        }
        if domain_id:
            limit['domain_id'] = domain_id
        post_body = json.dumps({'limits': [limit]})
        resp, body = self.post('limits', post_body)
        self.expected_success(201, resp.status)
        return rest_client.ResponseBody(resp, json.loads(body))

    def update_limit(self, limit_id, resource_limit, description=None):
        """Updates a limit in keystone by id."""

        limit = {'resource_limit': resource_limit}
        if description:
            limit['description'] = description
        patch_body = json.dumps({'limit': limit})
        resp, body = self.patch('limits/%s' % limit_id, patch_body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, json.loads(body))
